# 
#   This file is part of Mantis, a Multivariate ANalysis Tool for Spectromicroscopy.
# 
#   Copyright (C) 2011 Mirna Lerotic, 2nd Look
#   http://2ndlookconsulting.com
#   License: GNU GPL v3
#
#   Mantis is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version.
#
#   Mantis is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details <http://www.gnu.org/licenses/>.


from __future__ import division
from __future__ import print_function

import numpy as np
import os
import re

verbose = 0

title = 'Ncb'
extension = ['*.ncb']
read_types = ['image', 'stack']
write_types = ['image', 'stack', 'results']


def identify(filename):
    try:
        basename, extension = os.path.splitext(filename)
        dat_fn = basename + '.dat'
        f = open(str(dat_fn), 'r')
        f.close()
        return True
    except:
        return False


def GetFileStructure(FileName):
    return None


# ----------------------------------------------------------------------
def read(filename, self, selection=None, *args, **kwargs):
    basename, extension = os.path.splitext(filename)
    dat_fn = basename + '.dat'

    f = open(str(dat_fn), 'r')
    lines = f.readlines()
    temp = lines[0].split()

    self.n_cols = int(temp[0])
    self.n_rows = int(temp[1])
    scale = float(temp[2])

    # print self.n_cols, self.n_rows, scale

    temp = lines[1].split()
    x_start = float(temp[0])
    x_stop = float(temp[1])

    temp = lines[2].split()
    y_start = float(temp[0])
    y_stop = float(temp[1])

    self.n_ev = int(lines[3])

    # print self.n_ev

    self.ev = np.zeros((self.n_ev))
    self.data_dwell = np.zeros((self.n_ev))
    filename_list = []
    for i in range(self.n_ev):
        self.ev[i] = float(lines[4 + i])

    # print self.ev

    for i in range(self.n_ev):
        self.ev[i] = float(lines[4 + i])

    try:
        for i in range(self.n_ev):
            temp = lines[4 + self.n_ev + i].split()

            filename_list.append(temp[0])
            self.data_dwell[i] = float(temp[2])
    except:
        self.data_dwell = np.ones((self.n_ev))

    f.close()

    if scale < 0:
        dataformat = float32
    else:
        dataformat = np.int16

    # print 'data format', dataformat

    f = open(str(filename), 'rb')
    big_array = np.fromfile(f, dataformat, self.n_cols * self.n_rows * self.n_ev)
    f.close()

    if scale > 0 and scale != 1:
        image_stack = big_array.astype(float) / scale
        print("data rescaled by ", 1. / scale)
    else:
        image_stack = big_array.astype(float)

    if (x_start > x_stop):
        image_stack = image_stack[::-1, :, :]
        t = x_start
        x_start = x_stop
        x_stop = t
        print('x data reversed')

    if (y_start > y_stop):
        image_stack = image_stack[:, ::-1, :]
        t = y_start
        y_start = y_stop
        y_stop = t
        print('y data reversed')

    xstep = (x_stop - x_start) / (self.n_cols - 1)
    self.x_dist = np.arange(x_start, x_stop + xstep, xstep)

    ystep = (y_stop - y_start) / (self.n_rows - 1)
    self.y_dist = np.arange(y_start, y_stop + ystep, ystep)

    self.absdata = np.empty((self.n_cols, self.n_rows, self.n_ev))

    self.absdata = np.reshape(image_stack, (self.n_cols, self.n_rows, self.n_ev), order='F')

    self.fill_h5_struct_from_stk()

    return


# ----------------------------------------------------------------------
def read_ncb4D(self, filenames):
    if verbose: print('First energy stack:', filenames[0])

    data_files = natural_sort(filenames)

    neng = len(data_files)
    if verbose: print('Number of energies ', neng)

    self.stack4D = []
    theta = []

    for i in range(neng):
        # Read the data
        thisstack, thistheta = read_ncb_data(self, data_files[i])

        # Check if we have negative angles, if yes convert to 0-360deg
        for ith in range(len(thistheta)):
            if thistheta[ith] < 0:
                thistheta[ith] = thistheta[ith] + 360

        self.stack4D.append(thisstack)
        theta.append(thistheta)

        dims = thisstack.shape

    self.stack4D = np.array(self.stack4D)
    self.stack4D = np.transpose(self.stack4D, axes=(1, 2, 0, 3))

    self.theta = theta[0]
    self.n_theta = len(theta)

    self.absdata = self.stack4D[:, :, :, 0]

    self.data_dwell = np.ones((neng))

    self.fill_h5_struct_from_stk()

    return


# ----------------------------------------------------------------------
def read_ncb_data(self, filename):
    basename, extension = os.path.splitext(filename)
    dat_fn = basename + '.dat'

    f = open(str(dat_fn), 'r')
    lines = f.readlines()
    temp = lines[0].split()

    self.n_cols = int(temp[0])
    self.n_rows = int(temp[1])
    scale = float(temp[2])

    if verbose:
        print('self.n_cols, self.n_rows, scale', self.n_cols, self.n_rows, scale)

    temp = lines[1].split()
    x_start = float(temp[0])
    x_stop = float(temp[1])

    temp = lines[2].split()
    y_start = float(temp[0])
    y_stop = float(temp[1])

    self.n_theta = int(lines[3])

    angles = np.zeros((self.n_theta))
    self.data_dwell = np.zeros((self.n_theta))
    filename_list = []
    for i in range(self.n_theta):
        angles[i] = float(lines[4 + i])

    for i in range(self.n_theta):
        angles[i] = float(lines[4 + i])

    try:
        for i in range(self.n_theta):
            temp = lines[4 + self.n_ev + i].split()

            filename_list.append(temp[0])
            self.data_dwell[i] = float(temp[2])
    except:
        self.data_dwell = np.ones((self.n_theta))

    f.close()

    if scale < 0:
        dataformat = float32
    else:
        dataformat = np.int16

    f = open(str(filename), 'rb')
    big_array = np.fromfile(f, dataformat, self.n_cols * self.n_rows * self.n_theta)
    f.close()

    if scale > 0 and scale != 1:
        image_stack = big_array.astype(float) / scale
        if verbose: print("data rescaled by ", 1. / scale)
    else:
        image_stack = big_array.astype(float)

    if (x_start > x_stop):
        image_stack = image_stack[::-1, :, :]
        t = x_start
        x_start = x_stop
        x_stop = t
        if verbose: print('x data reversed')

    if (y_start > y_stop):
        image_stack = image_stack[:, ::-1, :]
        t = y_start
        y_start = y_stop
        y_stop = t
        if verbose: print('y data reversed')

    xstep = (x_stop - x_start) / (self.n_cols - 1)
    self.x_dist = np.arange(x_start, x_stop + xstep, xstep)

    ystep = (y_stop - y_start) / (self.n_rows - 1)
    self.y_dist = np.arange(y_start, y_stop + ystep, ystep)

    image_stack = np.reshape(image_stack, (self.n_cols, self.n_rows, self.n_theta), order='F')

    return image_stack, angles


# ----------------------------------------------------------------------
#  This procedure writes  a whole stack (3d (E,x,y) array) to a binary file
#  with associated *.dat file to track paramaters
def write(filename, stack, data_type):  # ,norm):
    #
    # def operate_on_Narray(A, B, function):
    #     try:
    #         return [operate_on_Narray(a, b, function) for a, b in zip(A, B)]
    #     except TypeError as e:
    #         # Not iterable
    #         return function(A, B)
    # import matplotlib.pyplot as plt
    # def export_figure_matplotlib(name,arr, dpi=100, resize_fact=1, plt_show=False):
    #     fig = plt.figure(frameon=False)
    #     fig.set_size_inches(arr.shape[1] / dpi, arr.shape[0] / dpi)
    #     ax = plt.Axes(fig, [0., 0., 1., 1.])
    #     ax.set_axis_off()
    #     fig.add_axes(ax)
    #     ax.imshow(arr, cmap='gray', interpolation='none')
    #     if plt_show:
    #         plt.show()
    #     else:
    #         plt.savefig(name+'.tif', dpi=(dpi * resize_fact))
    #         plt.close()
    print('Writing .ncb stack:', filename)

    basename, extension = os.path.splitext(filename)
    dat_fn = basename + '.dat'
    filename = basename + '.ncb'

    image_stack = np.transpose(stack.absdata, axes=(1, 0, 2))
    '''Experimental OD filtering and edge substraction'''
    # if norm:
    #
    #     image_stack = np.transpose(stack.od3d, axes=(1,0,2))
    #     #image_stack = stack.od3d.copy()
    #     OD_filter = np.amax(image_stack, 2)
    #     #returning the ODmask as tif
    #     mask = np.where((OD_filter > 2.5), 0, np.ones(np.shape(image_stack[:,:,0])))
    #     export_figure_matplotlib(basename+"od_mask",mask)
    #
    #     for i in range(np.size(image_stack, 2)):
    #         image_stack[:,:,i] = np.where((OD_filter > 2.5),0, image_stack[:,:,i])
    #     #image_stack = np.where(image_stack > 2,0, image_stack) #OD > 2 filter
    #     e0 = 0
    #     num = 5
    #     ##pre-edge
    #     ave = image_stack[:,:,e0]
    #     for i in range(num-1):
    #         ave = ave + image_stack[:,:,e0+i+1]
    #     ave = ave / num
    #
    #     #export pre-edge_average for contour plot.
    #     pre = np.transpose(stack.absdata, axes=(1,0,2))[:,:,e0]
    #     for i in range(num-1):
    #         pre = pre + np.transpose(stack.absdata, axes=(1,0,2))[:,:,e0+i+1]
    #     pre = pre / num
    #     export_figure_matplotlib(basename,pre)
    #
    #     for i in range(np.size(image_stack, 2)):
    #         image_stack[:,:,i] = image_stack[:,:,i]-ave
    #
    #     #post-edge
    #     ave = image_stack[:,:,-1]
    #     for i in range(num-1):
    #         ave = ave + image_stack[:,:,e0-i-2]
    #     ave = (ave / num) + 0.0001
    #     #print(ave)
    #     for i in range(np.size(image_stack, 2)):
    #         image_stack[:,:,i] = image_stack[:,:,i] - operate_on_Narray(ave, image_stack[:,:,i], lambda a, b: a/((1+ np.exp(-b+0.5*a)**(25*1/a))))

    image_stack = np.reshape(image_stack, (stack.n_cols * stack.n_rows * stack.n_ev), order='F')

    tmax = np.amax(image_stack)
    tmin = np.amin(image_stack)
    test = np.amax([abs(tmin), abs(tmax)])
    scale = 1.
    if test > 3e4 or test < 1e3:
        scale = 10. ** (3 - np.fix(np.math.log10(test)))

    # Save image stack to a binary .dat file
    f = open(str(filename), 'wb')
    if scale != 1.0:
        print('Scaling the data by ', scale)
        saveddata = image_stack * scale
        saveddata.astype(np.int16).tofile(f)
    else:
        scale = -1.0
        image_stack.astype(float32).tofile(f)
    f.close()

    # print 'imagedims', image_stack.shape

    f = open(str(dat_fn), 'w')

    print('\t%d\t%d\t%.6f' % (stack.n_rows, stack.n_cols, scale), file=f)

    x_start = stack.y_dist[0]
    x_stop = stack.y_dist[-1]
    if x_start != 0.:
        x_stop = x_stop - x_start
        x_start = 0.

    print('\t%.6f\t%.6f' % (x_start, x_stop), file=f)

    y_start = stack.x_dist[0]
    y_stop = stack.x_dist[-1]
    if y_start != 0.:
        y_stop = y_stop - y_start
        y_start = 0.
    print('\t%.6f\t%.6f' % (y_start, y_stop), file=f)

    print('\t%d' % (stack.n_ev), file=f)

    for i in range(stack.n_ev):
        print('\t%.6f' % (stack.ev[i]), file=f)

    for i in range(stack.n_ev):
        thisstr = 'image' + str(i + 1)
        print('%s\t%.6f\t%.6f' % (thisstr, stack.ev[i], stack.data_dwell[i]), file=f)

    f.close()

    return


# ----------------------------------------------------------------------
def natural_sort(l):
    convert = lambda text: int(text) if text.isdigit() else text.lower()
    alphanum_key = lambda key: [convert(c) for c in re.split('([0-9]+)', key)]
    return sorted(l, key=alphanum_key)


# ----------------------------------------------------------------------
class Cncb:
    def __init__(self):
        pass

    # ----------------------------------------------------------------------
    def read_ncb(self, filename):

        print('Reading stack:', filename)

        basename, extension = os.path.splitext(filename)
        dat_fn = basename + '.dat'

        f = open(str(dat_fn), 'r')
        lines = f.readlines()
        temp = lines[0].split()

        self.n_cols = int(temp[0])
        self.n_rows = int(temp[1])
        scale = float(temp[2])

        # print self.n_cols, self.n_rows, scale

        temp = lines[1].split()
        x_start = float(temp[0])
        x_stop = float(temp[1])

        temp = lines[2].split()
        y_start = float(temp[0])
        y_stop = float(temp[1])

        self.n_ev = int(lines[3])

        # print self.n_ev

        self.ev = np.zeros((self.n_ev))
        self.data_dwell = np.zeros((self.n_ev))
        filename_list = []
        for i in range(self.n_ev):
            self.ev[i] = float(lines[4 + i])

        # print self.ev

        for i in range(self.n_ev):
            self.ev[i] = float(lines[4 + i])

        try:
            for i in range(self.n_ev):
                temp = lines[4 + self.n_ev + i].split()

                filename_list.append(temp[0])
                self.data_dwell[i] = float(temp[2])
        except:
            self.data_dwell = np.ones((self.n_ev))

        f.close()

        if scale < 0:
            dataformat = float32
        else:
            dataformat = np.int16

        # print 'data format', dataformat

        f = open(str(filename), 'rb')
        big_array = np.fromfile(f, dataformat, self.n_cols * self.n_rows * self.n_ev)
        f.close()

        if scale > 0 and scale != 1:
            image_stack = big_array.astype(float) / scale
            print("data rescaled by ", 1. / scale)
        else:
            image_stack = big_array.astype(float)

        if (x_start > x_stop):
            image_stack = image_stack[::-1, :, :]
            t = x_start
            x_start = x_stop
            x_stop = t
            print('x data reversed')

        if (y_start > y_stop):
            image_stack = image_stack[:, ::-1, :]
            t = y_start
            y_start = y_stop
            y_stop = t
            print('y data reversed')

        xstep = (x_stop - x_start) / (self.n_cols - 1)
        self.x_dist = np.arange(x_start, x_stop + xstep, xstep)

        ystep = (y_stop - y_start) / (self.n_rows - 1)
        self.y_dist = np.arange(y_start, y_stop + ystep, ystep)

        self.absdata = np.empty((self.n_cols, self.n_rows, self.n_ev))

        self.absdata = np.reshape(image_stack, (self.n_cols, self.n_rows, self.n_ev), order='F')

        return

    # ----------------------------------------------------------------------
    #  This procedure writes  a whole stack (3d (E,x,y) array) to a binary file
    #  with associated *.dat file to track paramaters
    def write_ncb(self, filename, data_struct):

        print('Writing .ncb stack:', filename)

        basename, extension = os.path.splitext(filename)
        dat_fn = basename + '.dat'

        image_stack = np.transpose(self.absdata, axes=(1, 0, 2))

        # image_stack = self.absdata.copy()

        image_stack = np.reshape(image_stack, (self.n_cols * self.n_rows * self.n_ev), order='F')

        tmax = np.amax(image_stack)
        tmin = np.amin(image_stack)
        test = np.amax([abs(tmin), abs(tmax)])
        scale = 1.
        if test > 3e4 or test < 1e3:
            scale = 10. ** (3 - np.fix(np.math.log10(test)))

        # Save image stack to a binary .dat file
        f = open(str(filename), 'wb')
        if scale != 1.0:
            print('Scaling the data by ', scale)
            saveddata = image_stack * scale
            saveddata.astype(np.int16).tofile(f)
        else:
            scale = -1.0
            image_stack.astype(float32).tofile(f)
        f.close()

        # print 'imagedims', image_stack.shape

        f = open(str(dat_fn), 'w')

        print('\t%d\t%d\t%.6f' % (self.n_rows, self.n_cols, scale), file=f)
        # print('\t%d\t%d\t%.6f' %(self.n_cols, self.n_rows, scale)

        x_start = self.y_dist[0]
        x_stop = self.y_dist[-1]
        if x_start != 0.:
            x_stop = x_stop - x_start
            x_start = 0.

        print('\t%.6f\t%.6f' % (x_start, x_stop), file=f)

        y_start = self.x_dist[0]
        y_stop = self.x_dist[-1]
        if y_start != 0.:
            y_stop = y_stop - y_start
            y_start = 0.
        print('\t%.6f\t%.6f' % (y_start, y_stop), file=f)

        print('\t%d' % (self.n_ev), file=f)

        for i in range(self.n_ev):
            print('\t%.6f' % (self.ev[i]), file=f)

        for i in range(self.n_ev):
            thisstr = 'image' + str(i + 1)
            print('%s\t%.6f\t%.6f' % (thisstr, self.ev[i], self.data_dwell[i]), file=f)

        f.close()

        return

    # ----------------------------------------------------------------------
    def read_ncb_data(self, filename):

        # print 'Reading stack:', filename

        basename, extension = os.path.splitext(filename)
        dat_fn = basename + '.dat'

        f = open(str(dat_fn), 'r')
        lines = f.readlines()
        temp = lines[0].split()

        self.n_cols = int(temp[0])
        self.n_rows = int(temp[1])
        scale = float(temp[2])

        if verbose:
            print('self.n_cols, self.n_rows, scale', self.n_cols, self.n_rows, scale)

        temp = lines[1].split()
        x_start = float(temp[0])
        x_stop = float(temp[1])

        temp = lines[2].split()
        y_start = float(temp[0])
        y_stop = float(temp[1])

        self.n_theta = int(lines[3])

        # print self.n_ev

        angles = np.zeros((self.n_theta))
        self.data_dwell = np.zeros((self.n_theta))
        filename_list = []
        for i in range(self.n_theta):
            angles[i] = float(lines[4 + i])

        # print self.ev

        for i in range(self.n_theta):
            angles[i] = float(lines[4 + i])

        try:
            for i in range(self.n_theta):
                temp = lines[4 + self.n_ev + i].split()

                filename_list.append(temp[0])
                self.data_dwell[i] = float(temp[2])
        except:
            self.data_dwell = np.ones((self.n_theta))

        f.close()

        if scale < 0:
            dataformat = float32
        else:
            dataformat = np.int16

        # print 'data format', dataformat

        f = open(str(filename), 'rb')
        big_array = np.fromfile(f, dataformat, self.n_cols * self.n_rows * self.n_theta)
        f.close()

        if scale > 0 and scale != 1:
            image_stack = big_array.astype(float) / scale
            if verbose: print("data rescaled by ", 1. / scale)
        else:
            image_stack = big_array.astype(float)

        if (x_start > x_stop):
            image_stack = image_stack[::-1, :, :]
            t = x_start
            x_start = x_stop
            x_stop = t
            if verbose: print('x data reversed')

        if (y_start > y_stop):
            image_stack = image_stack[:, ::-1, :]
            t = y_start
            y_start = y_stop
            y_stop = t
            if verbose: print('y data reversed')

        xstep = (x_stop - x_start) / (self.n_cols - 1)
        self.x_dist = np.arange(x_start, x_stop + xstep, xstep)

        ystep = (y_stop - y_start) / (self.n_rows - 1)
        self.y_dist = np.arange(y_start, y_stop + ystep, ystep)

        image_stack = np.reshape(image_stack, (self.n_cols, self.n_rows, self.n_theta), order='F')

        return image_stack, angles

    # ----------------------------------------------------------------------
    def read_ncb4D(self, filenames):

        if verbose: print('First energy stack:', filenames[0])

        data_files = natural_sort(filenames)

        neng = len(data_files)
        if verbose: print('Number of energies ', neng)

        self.stack4D = []
        theta = []

        for i in range(neng):
            # Read the data
            thisstack, thistheta = self.read_ncb_data(data_files[i])

            # Check if we have negative angles, if yes convert to 0-360deg
            for ith in range(len(thistheta)):
                if thistheta[ith] < 0:
                    thistheta[ith] = thistheta[ith] + 360

            self.stack4D.append(thisstack)
            theta.append(thistheta)

            dims = thisstack.shape

        self.stack4D = np.array(self.stack4D)
        self.stack4D = np.transpose(self.stack4D, axes=(1, 2, 0, 3))

        self.theta = theta[0]
        self.n_theta = len(theta)

        self.absdata = self.stack4D[:, :, :, 0]

        self.data_dwell = np.ones((neng))

        return



# 
#   This file is part of Mantis, a Multivariate ANalysis Tool for Spectromicroscopy.
# 
#   Copyright (C) 2015 Mirna Lerotic, 2nd Look
#   http://2ndlookconsulting.com
#   License: GNU GPL v3
#
#   Mantis is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version.
#
#   Mantis is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details <http://www.gnu.org/licenses/>.

from __future__ import division
from __future__ import absolute_import

import os
import numpy as np
import scipy as sp
from time import time

import multiprocessing
from functools import partial

try:
    from skimage.transform import iradon, radon
except:
    print ('SIRT reconstruction requires skimage library.')

import warnings
warnings.simplefilter('ignore', DeprecationWarning)

from .TomoCS.forward_backward_tv import fista_tv,gfb_tv, gfb_tv_weng
from .TomoCS.projections import build_projection_operator
from .TomoCS.util import generate_synthetic_data

from .TomoCS import sirt as st

from . import Mrc
        
#----------------------------------------------------------------------
def write_mrc(stack, mrcfn):
    
    #stackf32 = stack.astype(np.float32)
    Mrc.save(stack, mrcfn,ifExists='overwrite',calcMMM=False)
    
    
#----------------------------------------------------------------------
def load_mrc( mrcfn):

    stack = Mrc.load(mrcfn)

    return stack

#----------------------------------------------------------------------
def calc_sirt(R, theta, Rmin, Rmax, nonnegconst, maxiter, nrows):

    R = (R-Rmin)/(Rmax-Rmin)

    S1 = np.sum(R)

    At = iradon(R, theta=theta, output_size=nrows)
    S2 = np.sum(At)
    At = (At/S2)*S1
    xk = At

    for k in range(maxiter):
        t = iradon(radon(xk,theta),theta)
        #normalize
        St = np.sum(t)
        t = (t/St)*S1
        #update using (At g - At A x_k)
        #new xk = xk + difference between reconstruction At_starting - t_previuous_step
        xk = xk + At - t

        if nonnegconst == 1:
            #delete values <0 aka not real!
            xk = xk.clip(min=0)

    return xk

#----------------------------------------------------------------------
def calc_cs(R, theta, Rmin, Rmax, l, beta, initx0, nonnegconst, maxiter):

    R = (R-Rmin)/(Rmax-Rmin)

    proj = R.ravel()[:, np.newaxis]
    proj_operator = build_projection_operator(l, n_dir=len(theta), angles=theta)

    res, engs = gfb_tv(proj, beta, maxiter, H=proj_operator, x0=initx0, nonnegconst=nonnegconst)

    return res[-1]

#----------------------------------------------------------------------
class Ctomo:
    def __init__(self, stkdata):

        self.stack = stkdata
        
        self.tomorec = []


#----------------------------------------------------------------------   
# Calculate tomo  reconstruction
# Algorithm = 0 : CS reconstruction for 1 dataset 
# Algorithm = 1 : SIRT reconstruction for 1 dataset 
    def calc_tomo(self, tomodata, theta, maxiter, beta, samplethickness, algorithm = 0,
                  x0=[], comp = 0, beta2 = 0, nonnegconst = 1, nprocessors=1):
        
        tomodata = tomodata
        #Algorithm 
        if algorithm == 0:
            if nprocessors <= 1:
                self.calc_tomo_cs(tomodata.astype(np.float32), theta, maxiter, beta, samplethickness,
                              nonnegconst = nonnegconst)
            else:
                self.calc_tomo_cs_multi(tomodata.astype(np.float32), theta, maxiter, beta, samplethickness,
                              nonnegconst = nonnegconst, nprocessors=nprocessors)
        elif algorithm == 1:
            self.calc_tomo_sirt(tomodata.astype(np.float32), theta, maxiter, beta, samplethickness, 
                                nonnegconst = nonnegconst, nprocessors=nprocessors)
        else:
            self.calc_tomo_cs_tveng(tomodata.astype(np.float32), theta, maxiter, beta, samplethickness,
                                    x0, comp, beta2, nonnegconst = nonnegconst)  
             
        
#----------------------------------------------------------------------   
# Calculate tomo - CS reconstruction for 1 dataset  with multiprocessing
    def calc_tomo_cs_multi(self, tomodata, theta, maxiter, beta, samplethickness, nonnegconst=1, nprocessors=6):
        

        print ('Compressed sensing TV regression')
        #print ('Angles ', theta)
        print ('TV beta ', beta)
        print ('MAX iterations ', maxiter)
        print ("Sample thickness ", samplethickness)
        
        #Check if we have negative angles, if yes convert to 0-360deg
        for i in range(len(theta)):
            if theta[i] < 0:
                theta[i] = theta[i] + 360
        #print ('Angles in 0-360 range: ', theta)
                      
        nang = len(theta)
        #print ('Number of angles ', nang)
                
        dims = tomodata.shape
        print ('Dimensions ', dims, tomodata.dtype)
    
        stack = np.swapaxes(tomodata, 0, 1)

        theta = np.deg2rad(theta)
        
        dims = stack.shape
        ncols = dims[0]
        nrows = dims[1]
        l = dims[1]

        Rmax = np.amax(stack)
        Rmin = np.amin(stack)

        recondata = []
        projections = []
        
        initx0 = np.zeros((nrows,nrows), dtype=np.float32)
        
        t1 = time()

        R = stack[int(ncols/2),:, :].T
        R = (R-Rmin)/(Rmax-Rmin)

        proj = R.ravel()[:, np.newaxis]
        l = dims[1]
        proj_operator = build_projection_operator(l, n_dir=len(theta), angles=theta)

        res, engs = gfb_tv(proj, beta, maxiter, H=proj_operator, x0=initx0, nonnegconst = nonnegconst)

        initx0 = res[-1]

        
        for j in range(ncols):

            projections.append(stack[j,:, :].T)
        
        Rmax = np.amax(stack)
        Rmin = np.amin(stack)

        partial_calc_cs = partial(calc_cs, theta=theta, Rmin=Rmin, Rmax=Rmax,
                                  l=l, beta=beta, initx0=initx0, nonnegconst=nonnegconst,
                                  maxiter=maxiter)
        pool = multiprocessing.Pool(processes=nprocessors)

        res = pool.map(partial_calc_cs, projections)
        pool.close()
        pool.join()

        recondata = res
            
        t2 = time()
        print ("reconstruction done in %f s" %(t2 - t1))
                
        #Save the 3D tomo reconstruction to a HDF5 file
        recondata=np.array(recondata)
        dims = recondata.shape
        print ('final dims', dims)
        
        #Crop the data is sample thickness is defined
        if (samplethickness > 0) and (samplethickness<dims[2]-2):
            recondata = recondata[:,:,dims[2]/2-samplethickness/2:dims[2]/2+samplethickness/2]
               
        
        recondata = np.swapaxes(recondata, 0, 1)
        
        self.tomorec = recondata
        

#         write_mrc(recondata, 'testMantistomo.mrc')
        
        return


#----------------------------------------------------------------------
# Calculate tomo - CS reconstruction for 1 dataset
    def calc_tomo_cs(self, tomodata, theta, maxiter, beta, samplethickness, nonnegconst = 1):


        print ('Compressed sensing TV regression')
        #print ('Angles ', theta)
        print ('TV beta ', beta)
        print ('MAX iterations ', maxiter)
        print ("Sample thickness ", samplethickness)

        #Check if we have negative angles, if yes convert to 0-360deg
        for i in range(len(theta)):
            if theta[i] < 0:
                theta[i] = theta[i] + 360
        #print ('Angles in 0-360 range: ', theta)

        nang = len(theta)
        #print ('Number of angles ', nang)

        dims = tomodata.shape
        print ('Dimensions ', dims, tomodata.dtype)

        stack = np.swapaxes(tomodata, 0, 1)


        theta = np.deg2rad(theta)

        dims = stack.shape
        ncols = dims[0]
        nrows = dims[1]

        Rmax = np.amax(stack)
        Rmin = np.amin(stack)

        recondata = []

        initx0 = np.zeros((nrows,nrows), dtype=np.float32)

        t1 = time()

        for j in range(ncols):
            print (j, ' of ', ncols)
        #             for j in range(1):
            #j=ncols/2
            R = stack[j,:, :].T
            R = (R-Rmin)/(Rmax-Rmin)

            proj = R.ravel()[:, np.newaxis]
            l = dims[1]
            proj_operator = build_projection_operator(l, n_dir=len(theta), angles=theta)

            # Reconstruction
            res, engs = gfb_tv(proj, beta, maxiter, H=proj_operator, x0=initx0, nonnegconst = nonnegconst)

            recondata.append(res[-1])
            initx0 = res[-1]

        t2 = time()
        print ("reconstruction done in %f s" %(t2 - t1))

        #Save the 3D tomo reconstruction to a HDF5 file
        recondata=np.array(recondata)
        dims = recondata.shape
        print ('final dims', dims)

        #Crop the data is sample thickness is defined
        if (samplethickness > 0) and (samplethickness<dims[2]-2):
            recondata = recondata[:,:,dims[2]/2-samplethickness/2:dims[2]/2+samplethickness/2]

        recondata = np.swapaxes(recondata, 0, 1)

        self.tomorec = recondata

        #write_mrc(recondata, 'CS_reconstruction.mrc')

        return


#----------------------------------------------------------------------   
# Calculate tomo - SIRT reconstruction for 1 dataset 
    def calc_tomo_sirt(self, tomodata, theta, maxiter, beta, samplethickness, nonnegconst = 1, nprocessors=6):
        
        try:
            from skimage.transform import iradon, radon
        except:
            return

        print ('SIRT')
        #print ('Angles ', theta)
        print ('MAX iterations ', maxiter)
        print ("Sample thickness ", samplethickness)
        
        #Check if we have negative angles, if yes convert to 0-360deg
        for i in range(len(theta)):
            if theta[i] < 0:
                theta[i] = theta[i] + 360
        #print ('Angles in 0-360 range: ', theta)
                      
        nang = len(theta)
        print( 'Number of angles ', nang)
    
        tomodata = np.swapaxes(tomodata, 0, 1)
                
        dims = tomodata.shape
        print ('Dimensions ', dims, tomodata.dtype)
     
        stack = tomodata.astype(np.float32)

        #theta = np.deg2rad(theta)
        
        dims = stack.shape
        ncols = dims[0]
        nrows = dims[1]

        t1 = time()

        print ('Calculating SIRT reconstruction')
        print ('Number of iterations: ', maxiter)
        #N Iterations
        error = []
        recondata = np.zeros((ncols, nrows, nrows))

        projections = []
        for j in range(ncols):
            projections.append(stack[j,:, :])

        Rmax = np.amax(stack)
        Rmin = np.amin(stack)

        partial_calc_sirt = partial(calc_sirt, theta=theta, Rmin=Rmin, Rmax=Rmax,
                                    nonnegconst=nonnegconst, maxiter=maxiter, nrows=nrows)
        pool = multiprocessing.Pool(processes=nprocessors)

        res = pool.map(partial_calc_sirt, projections)
        pool.close()
        pool.join()

        recondata = res
            
        t2 = time()
        print ("reconstruction done in %f s" %(t2 - t1))

        recondata = np.array(recondata)

        dims = recondata.shape

        #Crop the data is sample thickness is defined
        if (samplethickness > 0) and (samplethickness<dims[2]-2):
            print ('Cropping the data')
            recondata = recondata[:,:,dims[2]/2-samplethickness/2:dims[2]/2+samplethickness/2]

        recondata = np.swapaxes(recondata, 0, 1)
        recondata = np.swapaxes(recondata, 0, 2)
           
        self.tomorec = recondata
        
        return

#----------------------------------------------------------------------
# Calculate tomo - SIRT reconstruction for 1 dataset
    def calc_tomo_sirt_singleprocessor(self, tomodata, theta, maxiter, beta, samplethickness, nonnegconst = 1):

        print ('SIRT')
        #print ('Angles ', theta)
        print ('MAX iterations ', maxiter)
        print ("Sample thickness ", samplethickness)

        #Check if we have negative angles, if yes convert to 0-360deg
        for i in range(len(theta)):
            if theta[i] < 0:
                theta[i] = theta[i] + 360
        #print ('Angles in 0-360 range: ', theta

        nang = len(theta)
        print ('Number of angles ', nang)

        tomodata = np.swapaxes(tomodata, 0, 1)

        dims = tomodata.shape
        print ('Dimensions ', dims, tomodata.dtype)

        stack = tomodata.astype(np.float32)

        #theta = np.deg2rad(theta)

        dims = stack.shape
        ncols = dims[0]
        nrows = dims[1]

        t1 = time()

        print ('Calculating SIRT reconstruction')
        print ('Number of iterations: ', maxiter)
        #N Iterations
        n = maxiter
        error = []
        recondata = np.zeros((ncols, nrows, nrows))

        Rmax = np.amax(stack)
        Rmin = np.amin(stack)

        for j in range(ncols):
        #for j in range(1):
            #j=ncols/2
            print( 'processing ', j,' of ', ncols)

            R = stack[j,:, :]
            # Normalize the sinogram
            R = (R-Rmin)/(Rmax-Rmin)

            S1 = np.sum(R)

            At = iradon(R, theta=theta, output_size=nrows)
            S2 = np.sum(At)
            At = (At/S2)*S1
            xk = At

            for k in range(n):
                t = iradon(radon(xk,theta),theta)
                #normalize
                St = np.sum(t)
                t = (t/St)*S1
                #update using (At g - At A x_k)
                #new xk = xk + difference between reconstruction At_starting - t_previuous_step
                xk = xk + At - t

                if nonnegconst == 1:
                    #delete values <0 aka not real!
                    xk = xk.clip(min=0)

            recondata[j,:,:] = xk.copy()

        t2 = time()
        print( "reconstruction done in %f s" %(t2 - t1))

        dims = recondata.shape

        #Crop the data is sample thickness is defined
        if (samplethickness > 0) and (samplethickness<dims[2]-2):
            print( 'Cropping the data')
            recondata = recondata[:,:,dims[2]/2-samplethickness/2:dims[2]/2+samplethickness/2]

        recondata = np.swapaxes(recondata, 0, 1)
        recondata = np.swapaxes(recondata, 0, 2)

        self.tomorec = recondata

        return

    

#----------------------------------------------------------------------   
# Calculate tomo - CS reconstruction for 1 dataset with energy TV regularization
    def calc_tomo_cs_tveng(self, tomodata, theta, maxiter, beta, samplethickness, initrecs, comp, beta2, nonnegconst = 1):
        

        print ('Compressed sensing TV regression with Energy Regularization')
        #print ('Angles ', theta)
        print ('TV beta ', beta)
        print ('TV beta2', beta2)
        print ('MAX iterations ', maxiter)
        print ("Sample thickness ", samplethickness)
        
        #Check if we have negative angles, if yes convert to 0-360deg
        for i in range(len(theta)):
            if theta[i] < 0:
                theta[i] = theta[i] + 360
        #print ('Angles in 0-360 range: ', theta)
                      
        nang = len(theta)
        #print ('Number of angles ', nang)
                
        dims = tomodata.shape
        #print ('Dimensions ', dims, tomodata.dtype  )
    
        stack = np.swapaxes(tomodata, 0, 1)


        theta = np.deg2rad(theta)
        
        dims = stack.shape
        ncols = dims[0]
        nrows = dims[1]
    

        recondata = []
        
        t1 = time()
        
        for j in range(ncols):
            #(print j, ' of ', ncols)
        #             for j in range(1):
            #j=ncols/2
            R = stack[j,:, :].T

 
            Rmax = np.amax(R)
            Rmin = np.amin(R)
         
            R = (R-Rmin)/(Rmax-Rmin)

#             pl.imshow(R, cmap=pl.cm.Greys_r)
#             pl.show()
                           
            proj = R.ravel()[:, np.newaxis]
            l = dims[1]
            proj_operator = build_projection_operator(l, n_dir=len(theta), angles=theta)
        
            
            if comp==0:
                xb=initrecs[comp+1][j]
                xa=initrecs[comp+2][j]
            elif comp == len(initrecs)-1:
                xb=initrecs[comp-2][j]
                xa=initrecs[comp-1][j]
            else:
                xb=initrecs[comp-1][j]
                xa=initrecs[comp+1][j]
            
            # Reconstruction            
            res, engs = gfb_tv_weng(proj, beta, maxiter, H=proj_operator, 
                                        x0=np.array(initrecs[comp][j]),
                                        xb=xb, xa=xa, beta2=beta2,
                                        nonnegconst = nonnegconst)

    
        
            recondata.append(res[-1])
            
        t2 = time()
        print ("reconstruction done in %f s" %(t2 - t1))
                
        #Save the 3D tomo reconstruction to a HDF5 file
        recondata=np.array(recondata)
        dims = recondata.shape
        #print ('final dims', dims)
        
        #Crop the data is sample thickness is defined
        if (samplethickness > 0) and (samplethickness<dims[2]-2):
            recondata = recondata[:,:,dims[2]/2-samplethickness/2:dims[2]/2+samplethickness/2]
               
        
        recondata = np.swapaxes(recondata, 0, 1)
        
        self.tomorec = recondata

        self.save_mrc(recondata, 'testMantistomo.mrc')
        
        return
    
        
#----------------------------------------------------------------------   
# Save mrc
    def save_mrc(self, path, data):  
        
        data = np.array(data, dtype=np.float32)  
        
        write_mrc(data, path)



# -*- coding: utf-8 -*-
#
#   This file is part of Mantis, a Multivariate ANalysis Tool for Spectromicroscopy.
#
#   Copyright (C) 2015 Benjamin Watts, Paul Scherrer Institut
#   License: GNU GPL v3
#
#   Mantis is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version.
#
#   Mantis is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details <http://www.gnu.org/licenses/>.


from __future__ import print_function
import json
import re, numpy, sys
from os.path import splitext, isfile
from collections import OrderedDict


title = 'JSON'
extension = ['*.json', '*.hdr']
read_types = ['spectrum','image','stack']
write_types = []
def identify(filename):
    try:
        if isfile(splitext(filename)[0] + '.json'):
            JS = JS_FileLoader(splitext(filename)[0] + '.json')
            return JS.num_regions>0 # return true if file contains at least one region
        else:
            return False
    except:
        return False

def GetFileStructure(FileName):
    JS = JS_FileLoader(FileName)
    if JS.num_regions<2 and JS.num_channels<2 and isfile(splitext(FileName)[0] + '.json'): #if json file exists, skip datachoicedialog
        print("JSON-file found.")
        return None # exit if only one choice
    D = OrderedDict()
    for i,R in enumerate(['Region_'+str(r) for r in range(JS.num_regions)]):
        D[R] = OrderedDict()
        D[R].definition = 'JSON'
        D[R].scan_type = JS.js['ScanDefinition']['Type']
        D[R].data_shape = JS.data_size[i]
        D[R].data_axes = [JS.js['ScanDefinition']['Regions'][1]['PAxis']['Name'],JS.js['ScanDefinition']['Regions'][1]['QAxis']['Name'],JS.js['ScanDefinition']['StackAxis']['Name']]
        for ch in range(1,JS.num_channels+1):
            D[R][JS.js['Channels'][ch]['Name']] = OrderedDict()
    return D



#----------------------------------------------------------------------
class JS_FileLoader:
  """Parse .hdr file for metadata."""
  js = []
  f = []
  def __init__(self, fileName,identify=False):
    fileName = (splitext(fileName)[0] + '.json')
    if not JS_FileLoader.js or JS_FileLoader.f != fileName: # prevent class from fetching and parsing the *.hdr file multiple times. Use the class attribute "hdr" instead if available and check if a new file is loaded.
        # Parse the JSON file
        with open(fileName) as f:
            JS_FileLoader.js = json.load(f)
        JS_FileLoader.f = fileName
    else:
        None
    self.js = JS_FileLoader.js
    if 'ScanDefinition' in self.js:
      self.num_regions = int(self.js['ScanDefinition']['Regions'][0])
      self.num_channels = int(self.js['Channels'][0])
      self.file_path, self.file_ext = splitext(fileName)
      self.data_size = self.parse_DataSize()
      self.data_names = self.parseDataNames()
    else:
      self.num_regions = 0
      self.num_channels = 0

#----------------------------------------------------------------------
  def parseDataNames(self): #ToDo: Assigning Scan types by DataFlags is prone to errors. Better detect automatically via (HDR.hdr['ScanDefinition']['Regions'][0]) etc.
    """Figure out names for the .xsp or .xim files that contain the actual data, then check that the files actually exist, printing warnings if they don't."""
    DataNames = []#Regions
    DataNames2 = []#Channels
    DataNames3 = []#Energies
    Alphabet = 'abcdefghijklmnopqrstuvwxyz'
    DataFlag = self.js['ScanDefinition']['Flags']
    if DataFlag in ['Spectra','Multi-Region Spectra']:
      for num_R in range(self.num_regions):
        DataNames2 = []
        for num_Ch in range(self.num_channels):
          DataNames2.append([self.file_path+'_'+str(num_R)+'.xsp'])
        DataNames.append(DataNames2)
    elif DataFlag == 'Image':
      DataNames2 = []
      for num_Ch in range(self.num_channels):
        DataNames2.append([self.file_path+'_'+Alphabet[num_Ch]+'.xim'])
      DataNames.append(DataNames2)
    elif DataFlag in ['Multi-Region Image']:
      for num_R in range(self.num_regions):
          DataNames2 = []
          for num_Ch in range(self.num_channels):
              DataNames2.append([self.file_path + '_' + Alphabet[num_Ch] + str(num_R) + '.xim'])
          DataNames.append(DataNames2)
    elif DataFlag in ['Multi-Region Image Stack']:
        for num_Ch in range(self.num_channels):
            DataNames2 = []
            for num_R in range(self.num_regions):
                DataNames3 = []
                for num_E in range(self.data_size[0][2]):
                    DataNames3.append(
                        self.file_path + '_' + Alphabet[num_Ch] + str(num_E).zfill(3) + str(num_R) + '.xim')
                DataNames2.append(DataNames3)
            DataNames = [DataNames2]
            # print(DataNames[0])
    elif DataFlag == 'Image Stack':
      DataNames2 = []
      for num_Ch in range(self.num_channels):
        DataNames3 = []
        for num_E in range(self.data_size[0][2]):
          DataNames3.append(self.file_path+'_'+Alphabet[num_Ch]+str(num_E).zfill(3)+'.xim')
        DataNames2.append(DataNames3)
      DataNames = [DataNames2]
    else:
      print("WARNING! Unknown flag:", DataFlag)
    #for num_R in range(len(DataNames)):#Check that names correspond to existing files
      #for num_Ch in range(len(DataNames[num_R])):
        #for num_E in range(len(DataNames[num_R][num_Ch])):
          #if exists(DataNames[num_R][num_Ch][num_E]) == False:
            #print "WARNING! Data file doesn't exist:", DataNames[num_R][num_Ch][num_E]
    return DataNames

#----------------------------------------------------------------------
  def parse_DataSize(self):
    """Calculate data array size. This is useful for making sure all of the lists of data are the correct length."""
    DataSize = []
    for R_num in range(self.num_regions):
      DataSize.append([1,1,1])# [PAxis,QAxis,StackAxis] (switch to [X1,X2,E] later)]
      DataSize[R_num][0] = int(self.js['ScanDefinition']['Regions'][R_num+1]['PAxis']['Points'][0])
      if 'QAxis' in self.js['ScanDefinition']['Regions'][R_num+1] and 'Points' in self.js['ScanDefinition']['Regions'][R_num+1]['QAxis']:
        DataSize[R_num][1] = int(self.js['ScanDefinition']['Regions'][R_num+1]['QAxis']['Points'][0])
      if 'StackAxis' in self.js['ScanDefinition'] and 'Points' in self.js['ScanDefinition']['StackAxis']:
        DataSize[R_num][2] = int(self.js['ScanDefinition']['StackAxis']['Points'][0])
      if self.js['ScanDefinition']['Type'] in ['NEXAFS Point Scan','NEXAFS Line Scan']:
        DataSize[R_num] = [DataSize[R_num][1],1,DataSize[R_num][0]]#switch to [X1,X2,E] format
#        DataSize[R_num] = [1,DataSize[R_num][1],DataSize[R_num][0]]#also works, but might be problematic for finding number of spatial points
    return DataSize

#----------------------------------------------------------------------
def read(filename, self, selection=None, *args, **kwargs):
    JS = JS_FileLoader(filename)
    allowed = ['Image Stack', 'Image', 'Multi-Region Image Stack']
    if JS.js['ScanDefinition']['Flags'] in allowed:
        self.x_dist = numpy.array([float(i) for i in JS.js['ScanDefinition']['Regions'][selection[0]+1]['PAxis']['Points'][1:] ])
        self.y_dist = numpy.array([float(i) for i in JS.js['ScanDefinition']['Regions'][selection[0]+1]['QAxis']['Points'][1:] ])
        self.ev = numpy.array([float(i) for i in JS.js['ScanDefinition']['StackAxis']['Points'][1:] ])
        self.n_cols = len(self.x_dist)
        self.n_rows = len(self.y_dist)
        self.n_ev = len(self.ev)

        msec = float(JS.js['ScanDefinition']['Dwell'])
        self.data_dwell = numpy.ones((self.n_ev))*msec

        imagestack = numpy.empty((self.n_cols,self.n_rows,self.n_ev), numpy.int32)
        for i in range(len(JS.data_names[selection[1]][selection[0]])):
            try:
                imagestack[:,:,i] = numpy.loadtxt(JS.data_names[selection[1]][selection[0]][i], numpy.int32).T
            except ValueError:
                print("Aborted stack or XIMs with inconsistent dimensions.")
                imagestack[:,:,i] = numpy.nan
            except IOError:
                print("Image file not found.")
                imagestack[:,:,i] = numpy.nan
        self.absdata = numpy.empty((self.n_cols,self.n_rows, self.n_ev))

        self.absdata = numpy.reshape(imagestack, (self.n_cols,self.n_rows, self.n_ev), order='F')

        self.fill_h5_struct_from_stk()

    else:
        print("Unknown Format")

    return

#----------------------------------------------------------------------
def read_js_i0(self, filename, *args, **kwargs):
    JS = JS_FileLoader(filename)

    if 'ScanType' in JS.js['ScanDefinition'] and JS.js['ScanDefinition']['ScanType'] == 'Spectra':
        Energies = JS.js['ScanDefinition']['Regions'][1]['PAxis']['Points'][1:]
        tempimage = numpy.loadtxt(JS.data_names[0][0][0], numpy.float32)
        Data = tempimage[:,1]
    elif JS.js['ScanDefinition']['Type'] == 'NEXAFS Line Scan':
        Energies = JS.js['ScanDefinition']['Regions'][1]['PAxis']['Points'][1:]
        tempimage = numpy.loadtxt(JS.data_names[0][0][0], numpy.int32)
        Data = numpy.mean(tempimage,axis=0)
    else:# Image Stack
        Energies = JS.js['ScanDefinition']['StackAxis']['Points'][1:]
        tempimage = numpy.empty((JS.data_size[0][0],JS.data_size[0][1]), numpy.int32)
        Data = numpy.empty((JS.data_size[0][2]), numpy.int32)
        for i in range(len(JS.data_names[0][0])):
            tempimage = numpy.loadtxt(JS.data_names[0][0][i], numpy.int32)
            Data[i] = numpy.mean(tempimage)


    msec = float(JS.js['ScanDefinition']['Dwell'])#shouldn't this be needed?
    self.i0_dwell = msec
    self.evi0 = numpy.array([float(i) for i in Energies])
    self.i0data = Data
    return


# 
#   This file is part of Mantis, a Multivariate ANalysis Tool for Spectromicroscopy.
# 
#   Copyright (C) 2016 Mirna Lerotic, 2nd Look
#   http://2ndlookconsulting.com
#   License: GNU GPL v3
#
#   Mantis is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version.
#
#   Mantis is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details <http://www.gnu.org/licenses/>.

from __future__ import division

import os
import numpy as np
#from skimage.morphology._greyreconstruct import reconstruction_loop

M_PI = 3.14159265358979323846264338327


#----------------------------------------------------------------------
def preprocessing(ry, rz, num_pixels, center, gridx, gridy) :

    for i in range(ry):
        gridx[i] = -ry/2.+i
    
    for i in range(rz):
        gridy[i] = -rz/2.+i

    mov = float(num_pixels)/2.0-center
    if (mov-np.ceil(mov) < 1e-2):
        mov += 1e-2
    
    return mov, gridx, gridy


#----------------------------------------------------------------------
def calc_quadrant(theta_p) :

    if ((theta_p >= 0 and theta_p < M_PI/2) or (theta_p >= M_PI and theta_p < 3*M_PI/2)) :
        quadrant = True
    
    else :
        quadrant = False
    
    return quadrant

#----------------------------------------------------------------------
def calc_coords(ry, rz, xi, yi, sin_p, cos_p,
                gridx, gridy, coordx, coordy):


    srcx = xi*cos_p-yi*sin_p
    srcy = xi*sin_p+yi*cos_p
    detx = -xi*cos_p-yi*sin_p
    dety = -xi*sin_p+yi*cos_p    

    slope = (srcy-dety)/(srcx-detx)
    islope = 1/slope
    for n in range(ry+1): 
        coordy[n] = slope*(gridx[n]-srcx)+srcy

    for n in range(rz+1) :
        coordx[n] = islope*(gridy[n]-srcy)+srcx
    

    return coordx, coordy

#----------------------------------------------------------------------
def trim_coords( ry, rz, coordx, coordy, 
                 gridx, gridy, ax, ay, 
                 bx, by):

    asize = 0
    bsize = 0
    for n in range(rz+1): 
        if (coordx[n] > gridx[0]) :
            if (coordx[n] < gridx[ry]) :
                ax[asize] = coordx[n]
                ay[asize] = gridy[n]
                asize +=1
            
    for n in range(ry+1): 
        if (coordy[n] > gridy[0]) :
            if (coordy[n] < gridy[rz]) :
                bx[bsize] = gridx[n]
                by[bsize] = coordy[n]
                bsize +=1

    return asize, ax, ay, bsize, bx, by


#----------------------------------------------------------------------
def sort_intersections( ind_condition, asize, ax, ay, 
                        bsize, bx, by, 
                        coorx, coory) :

    i=0
    j=0
    k=0

    while (i<asize and j<bsize):
        a_ind = i if ind_condition else (asize-1-i)
        if (ax[a_ind] < bx[j]) :
        
            coorx[k] = ax[a_ind]
            coory[k] = ay[a_ind]
            i += 1
            k += 1
        
        else :
        
            coorx[k] = bx[j]
            coory[k] = by[j]
            j += 1
            k +=1
        
    
    while (i < asize) :
        a_ind = i if ind_condition else (asize-1-i)
        coorx[k] = ax[a_ind]
        coory[k] = ay[a_ind]
        i += 1
        k += 1
    
    while (j < bsize) :
        coorx[k] = bx[j]
        coory[k] = by[j]
        j += 1
        k += 1
    
    
    csize = asize+bsize
    
    return csize


#----------------------------------------------------------------------
def calc_dist( ry, rz, csize, coorx, coory, indi, dist):

    for n in range(csize-1): 
    
        diffx = coorx[n+1]-coorx[n]
        diffy = coory[n+1]-coory[n]
        dist[n] = np.sqrt(diffx*diffx+diffy*diffy)
        midx = (coorx[n+1]+coorx[n])/2
        midy = (coory[n+1]+coory[n])/2
        x1 = midx+ry/2.
        x2 = midy+rz/2.
        i1 = (int)(midx+ry/2.)
        i2 = (int)(midy+rz/2.)
        indx = i1-(i1>x1)
        indy = i2-(i2>x2)
        indi[n] = indy+(indx*rz)
    
    return indi, dist



#----------------------------------------------------------------------
def calc_simdata( p, s, c, ry, rz, num_slices, num_pixels, 
                  csize, indi, dist, model, simdata):

    index_model = s*ry*rz
    index_data = c+s*num_pixels+p*num_slices*num_pixels

    for n in range(csize-1): 
        simdata[index_data] += model[indi[n]+index_model]*dist[n]
        #simdata[index_data] += model[p,s,c]*dist[n]
    
    return simdata




    
#----------------------------------------------------------------------   
#     Calculate sirt
#     tomo : ndarray
#         3D tomographic data.
#     theta : array
#         Projection angles in radian.
#     recon : ndarray, optional
#         Initial values of the reconstruction object.
#     num_gridx, num_gridy : int, optional
#         Number of pixels along x- and y-axes in the reconstruction grid.
#     num_iter : int, optional
#         Number of algorithm iterations performed.
def calculate_sirt(tomo, theta, num_iter):
    
    dx, dy, dz = tomo.shape
    print ('Calculate SIRT reconstruction')
    center = np.ones(dy, dtype='float32') * dz / 2.
    ngridx = dz
    ngridy = dz
    #tomo = -np.log(tomo)
    #recon = 1e-6 * np.ones((dy, ngridx, ngridy), dtype='float32')
    recon = 1e-6 * np.ones((dy*ngridx*ngridy), dtype='float32')
    
    print ('tomoshape', tomo.shape)
    
        
    data = tomo
    
    gridx = np.zeros((ngridx+1), dtype='float32')
    gridy = np.zeros((ngridy+1), dtype='float32')

    coordx = np.zeros((ngridy+1), dtype='float32')
    coordy = np.zeros((ngridx+1), dtype='float32') 
    
    ax = np.zeros((ngridx+ngridy), dtype='float32')    
    ay = np.zeros((ngridx+ngridy), dtype='float32') 
    bx = np.zeros((ngridx+ngridy), dtype='float32')       
    by = np.zeros((ngridx+ngridy), dtype='float32') 

    coorx = np.zeros((ngridx+ngridy), dtype='float32') 
    coory = np.zeros((ngridx+ngridy), dtype='float32') 
    dist = np.zeros((ngridx+ngridy), dtype='float32') 
    indi = np.zeros((ngridx+ngridy), dtype='int') 

          
    for i in range(num_iter): 
        print ('Iteration ', i)
        simdata = np.zeros((dx*dy*dz), dtype='float32')

        #For each slice
        for s in range(dy): 
            print ('Slice', s)

            mov, gridx, gridy = preprocessing(ngridx, ngridy, dz, center[s], gridx, gridy) 

            sum_dist = np.zeros((ngridx*ngridy), dtype='float32') 
            update = np.zeros((ngridx*ngridy), dtype='float32') 
            
            # For each projection angle 
            for p in range(dx): 
                # Calculate the sin and cos values 
                # of the projection angle and find
                # at which quadrant on the cartesian grid.
                theta_p = np.fmod(theta[p], 2*M_PI)
                quadrant = calc_quadrant(theta_p)
                sin_p = np.sin(theta_p)
                cos_p = np.cos(theta_p)
    
                # For each detector pixel 
                for d in range(dz): 
                
                    # Calculate coordinates
                    xi = -1e6
                    yi = -(dz-1)/2.0+d+mov
                    coordx, coordy = calc_coords(
                        ngridx, ngridy, xi, yi, sin_p, cos_p, gridx, gridy, 
                        coordx, coordy)

                    # Merge the (coordx, gridy) and (gridx, coordy)
                    asize, ax, ay, bsize, bx, by = trim_coords(
                        ngridx, ngridy, coordx, coordy, gridx, gridy, 
                        ax, ay, bx, by)

                    # Sort the array of intersection points (ax, ay) and
                    # (bx, by). The new sorted intersection points are 
                    # stored in (coorx, coory). Total number of points 
                    # are csize.
                    csize = sort_intersections(
                        quadrant, asize, ax, ay, bsize, bx, by, 
                        coorx, coory)

                    # Calculate the distances (dist) between the 
                    # intersection points (coorx, coory). Find the 
                    # indices of the pixels on the reconstruction grid.
                    indi, dist = calc_dist(
                        ngridx, ngridy, csize, coorx, coory, 
                        indi, dist)

                    # Calculate simdata 
                    simdata = calc_simdata(p, s, d, ngridx, ngridy, dy, dz,
                        csize, indi, dist, recon, simdata)

                    # Calculate dist*dist
                    sum_dist2 = 0.0
                    for n in range(csize-1): 
                        sum_dist2 += dist[n]*dist[n]
                        sum_dist[indi[n]] += dist[n]
                    

                    # Update
                    if (sum_dist2 != 0.0) :
                    
                        ind_data = d+s*dz+p*dy*dz
                        upd = (data[p,s,d]-simdata[ind_data])/sum_dist2
                        for n in range(csize-1): 
                            update[indi[n]] += upd*dist[n]
                
            m = 0
            for n in range(ngridx*ngridy):
                if (sum_dist[n] != 0.0) :
                    ind_recon = s*ngridx*ngridy
                    recon[m+ind_recon] += update[m]/sum_dist[n]
                m+=1
                           

    recon = np.reshape(recon,(dy, ngridx, ngridy), order='C')
    
    print ('SIRT Reconstruction Done')
    return recon
        
            


import numpy as np
import scipy as sp
from scipy.integrate import trapz
import sys, time

from . import analyze

class nnma():

    def __init__(self, stkdata):

        self.stack = stkdata
    
        # Set default values for various NNMA parameters
        self.kNNMA = 5            # use no. of significant PCA components, otherwise default = 5              
        self.maxIters = 10         # default no. of iterations
        self.deltaErrorThreshold = 1e-3        # default threshold for change in reconstruction error
        self.initMatrices = 'Random'  # default initialization for matrices
    
        self.lambdaSparse = 0.        # sparseness regularization param
        self.lambdaClusterSim = 0.    # cluster spectra similarity regularization param
        self.lambdaSmooth = 0.       # smoothness regularization param


    # Fill initial matrices depending on user-specified initialization method
    def setParameters(self, kNNMA = 5, 
                            maxIters = 10,
                            deltaErrorThreshold = 1e-3,
                            initMatrices = 'Random',
                            lambdaSparse = 0.,
                            lambdaClusterSim = 0.,
                            lambdaSmooth = 0.):
        
        # Set the values for various NNMA parameters
        self.kNNMA = kNNMA                  
        self.maxIters = maxIters         
        self.deltaErrorThreshold = deltaErrorThreshold        
        self.initMatrices = initMatrices  
    
        self.lambdaSparse = lambdaSparse       
        self.lambdaClusterSim = lambdaClusterSim    
        self.lambdaSmooth = lambdaSmooth      


    def setClusterSpectra(self, clusterspectra):
        
        self.clusterspectra = clusterspectra.T.copy() 
        
        
    def setStandardsSpectra(self, standspectra):
        
        self.standspectra = standspectra

#---------------------------------------------------------------------------------------
# Define some helper functions for NNMA
#---------------------------------------------------------------------------------------

    # Fill initial matrices depending on user-specified initialization method
    def fillInitMatrices(self):
        print ('Init method:', self.initMatrices)
        if self.initMatrices == 'Random':
            muInit = np.random.rand(self.nEnergies, self.kNNMA)
            tInit = np.random.rand(self.kNNMA, self.nPixels)
        elif self.initMatrices == "Cluster":
            muInit = self.clusterspectra
            tInit = np.random.rand(self.kNNMA, self.nPixels)  # use SVD on mu to find t instead?
        elif self.initMatrices == "Standards":
            muInit = self.standspectra
            tInit = np.random.rand(self.kNNMA, self.nPixels)  # use SVD on mu to find t instead?
        return muInit, tInit

    # Update t
    def tUpdate(self, mu, t):
        tUpdateFactor = np.dot(mu.T, self.OD) / ( np.dot(mu.T, np.dot(mu, t)) + self.lambdaSparse + 1e-9 )
        tUpdated = t * tUpdateFactor
        return tUpdated

    # Update mu
    def muUpdate(self, mu, tUpdated):
        dCostSmooth_dMu = self.calcDCostSmooth_dMu(mu)
        muDiff = mu - self.muCluster
        muUpdateFactor = np.dot(self.OD, tUpdated.T) / ( np.dot(mu, np.dot(tUpdated, tUpdated.T))
                         + self.lambdaSmooth*dCostSmooth_dMu + self.lambdaClusterSim*2*muDiff + 1e-9 )
        muUpdated = mu * muUpdateFactor
        # Normalize each column of mu
        return muUpdated
  
    # Calculate sparseness contribution (JSparse) to cost function
    def calcCostSparse(self, t):
        costSparse = np.sum(np.sum(np.abs(t)))
        return costSparse

    # Calculate cluster spectra similarity contribution (JClusterSim) to cost function
    def calcCostClusterSim(self, mu):
        muDiff = mu - self.muCluster
        costClusterSim = (np.linalg.norm(muDiff))**2 
        return costClusterSim

    # Calculate smoothness contribution (JSmooth) to cost function
    def calcCostSmooth(self, mu):
        return 0.
  
    # Calculate dJSmooth/dMu needed in mu update algorithm
    def calcDCostSmooth_dMu(self, mu):
        return 0.
 
    # Calculate integral of each column in mu for normalization
    def calcMuColNorm(self, mu, muRefNorm):
        for k in range(self.kNNMA):
            muNorm = trapz(mu[:, k], x=self.energies)
            mu[:, k] =  (mu[:, k] / muNorm) * muRefNorm[k]
        return mu
 
    # Calculate current total cost function, fill cost function array
    def calcCostFn(self, mu, t, count):
        D = np.dot(mu, t)
        costDataMatch = 0.5 * (np.linalg.norm(self.OD - D))**2
        costSparse = self.calcCostSparse(t)
        costClusterSim = self.calcCostClusterSim(mu)
        costSmooth = self.calcCostSmooth(mu)
        costTotal = ( costDataMatch + self.lambdaSparse*costSparse 
		  + self.lambdaClusterSim*costClusterSim + self.lambdaSmooth*costSmooth )
        if count > 0:
            deltaError = self.costFnArray[count, 1] - self.costFnArray[count-1, 1]
        elif count == 0:
            deltaError = -1e-13
        self.costFnArray[count, :] = np.array([costTotal, deltaError, 
	          costSparse, costClusterSim, costSmooth])
        return costTotal, deltaError


#---------------------------------------------------------------------------------------
# Calculate NNMA
#---------------------------------------------------------------------------------------
    def calcNNMA(self, initmatrices = 'Random'):
      
        print ('calculating nnma')
        
        self.initMatrices = initmatrices

        self.OD = self.stack.od.copy()
        self.energies = self.stack.ev
        self.nEnergies = self.stack.n_ev
        self.nCols = self.stack.n_cols
        self.nRows = self.stack.n_rows
        self.nPixels = self.nCols * self.nRows
    
        # Transpose optical density matrix since NNMA needs dim NxP
        self.OD = self.OD.T
        # Zero out negative values in OD matrix
        negInd = np.where(self.OD < 0.)
        if negInd: self.OD[negInd] = 0.

        self.muRecon = np.zeros((self.nEnergies, self.kNNMA))
        self.tRecon = np.zeros((self.kNNMA, self.nPixels))
        self.DRecon = np.zeros((self.nEnergies, self.nPixels))
        self.costFnArray = np.zeros((self.maxIters+1, 5))
        self.costFnArray[0, 0] = 1e99
        self.costTotal = 0.    # stores current value of total cost function
        self.deltaError = 0    # stores current value of cost function change

        # If doing cluster spectra similarity regularization, import cluster spectra
        if initmatrices == 'Cluster':
            self.muCluster = self.clusterspectra
        else:
            self.muCluster = 0.

        self.timeTaken = 0.    # stores time taken to complete NNMA analysis

        # Initialize matrices
        muInit, tInit = self.fillInitMatrices()
        self.muinit = muInit.copy()
        muCurrent = muInit
        tCurrent = tInit
        count = 0
        costCurrent, deltaErrorCurrent = self.calcCostFn(muCurrent, tCurrent, count)

        # Start NNMA
        startTime = time.time()
  
        while ((count < self.maxIters) and (self.deltaError < self.deltaErrorThreshold)):
            # Store values from previous iterations before updating
            self.tRecon = tCurrent
            self.muRecon = muCurrent
            self.costTotal = costCurrent
            self.deltaError = deltaErrorCurrent

            # Now do NNMA update
            tUpdated = self.tUpdate(muCurrent, tCurrent)
            muUpdated = self.muUpdate(muCurrent, tUpdated)
            # Zero out any negative values in t and mu
            negIndT = np.where(tUpdated < 0.)
            if negIndT: tUpdated[negIndT] = 0.
            negIndMu = np.where(muUpdated < 0.)
            if negIndMu: muUpdated[negIndMu] = 0.
            tCurrent = tUpdated
            muCurrent = muUpdated

            # Calculate cost function
            costCurrent, deltaErrorCurrent = self.calcCostFn(muCurrent, tCurrent, count)

            count = count + 1
            print ('Iteration number {0}/{1}'.format(count,self.maxIters))

        endTime = time.time()
        self.timeTaken = endTime - startTime
        if count < self.maxIters:
            self.maxIters = count
            
        self.tRecon = np.reshape(self.tRecon, (self.kNNMA, self.nCols, self.nRows), order='F')
        
        
        return 1



# 
#   This file is part of Mantis, a Multivariate ANalysis Tool for Spectromicroscopy.
# 
#   Copyright (C) 2011 Mirna Lerotic, 2nd Look
#   http://2ndlookconsulting.com
#   License: GNU GPL v3
#
#   Mantis is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version.
#
#   Mantis is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details <http://www.gnu.org/licenses/>.


'''
! This is from file COMPOUND.DAT for HENKE.  Entries are:
! compound    formula                    density (g/cc)
water           H2O                                 1.0
protein         H48.6C32.9N8.9O8.9S0.6              1.35
lipid           H62.5C31.5O6.3                      1.0
nucleosome      H42.1C31.9N10.3O13.9P1.6S0.3        1.5
dna             H35.5C30.8N11.7O18.9P3.1            1.7
helium          He                                  0.1663e-3
chromatin       H49.95C24.64N8.66O15.57P1.07S0.03   1.527
air             N78.08O20.95Ar0.93                  1.2047e-3
pmma            C5H8O2                              1.18
nitride         Si3N4                               3.44
graphite        C                                   2.26
nickel          Ni                                  8.876
beryl           Be                                  1.845
copper          Cu                                  8.96
quartz          SiO2                                2.20
aluminum        Al                                  2.70
gold            Au                                  19.3
ice             H2O                                 0.92
carbon          C                                   1.0
polystyrene     C8H8                                1.06
silicon         Si                                  2.33
germanium       Ge                                  5.323
'''
from __future__ import division

import numpy as np
import scipy.interpolate
from xdrlib import *
import string

#-----------------------------------------------------------------------------
class henke:
    def __init__(self):   
        
        self.compound_name = [ 'water' , 'protein', 'lipid', 'nucleosome', 'dna', 'helium', 'chromatin', 
                               'air', 'pmma', 'nitride', 'graphite', 'nickel', 'beryl', 'copper', 
                               'quartz', 'aluminum', 'gold', 'ice', 'carbon', 'polystyrene', 
                               'silicon', 'germanium']
        
        self.compound_forumula = ['H2O', 'H48.6C32.9N8.9O8.9S0.6', 'H62.5C31.5O6.3', 
                                   'H42.1C31.9N10.3O13.9P1.6S0.3', 'H35.5C30.8N11.7O18.9P3.1',
                                   'He' , 'H49.95C24.64N8.66O15.57P1.07S0.03', 
                                   'N78.08O20.95Ar0.93', 'C5H8O2', 'Si3N4', 'C', 'Ni', 'Be', 
                                   'Cu', 'SiO2', 'Al', 'Au', 'H2O', 'C', 'C8H8',
                                   'Si', 'Ge']
        self.compound_density = [ 1.0, 1.35, 1.0, 1.5, 1.7, 1.66E-04, 1.527, 1.20E-03, 1.18,
                                 3.44, 2.26, 8.876, 1.845, 8.96, 2.2, 2.7, 19.3, 0.92, 1, 1.06,
                                 2.33, 5.323 ]



#-----------------------------------------------------------------------------    
    def compound(self, compound_string, density): 
        
#    Type in a chemical formula and the density.  The routine
#    then figures out the atomic weight in g/mole of "molecules"
#    and how many of what Z atoms you have.   

        z_array = []
        
        
        if compound_string in self.compound_name:
            compound_string = self.compound_forumula[self.compound_name.index(compound_string)] 

        if compound_string in self.compound_forumula: 
            z_array = self.zcompound(compound_string, z_array)
            atwt =  self.zatwt(z_array)

        return z_array, atwt
            
#-----------------------------------------------------------------------------               
    def zcompound(self, compound_string, z_array, paren_multiplier=False):
        
        verbose = False
        if verbose: print ('compound_string', compound_string)
        if paren_multiplier == False:
            z_array = np.zeros(92)
            paren_multiplier=1.

      
        max_z_index=93
        
        last_char_index = len(compound_string) - 1
        
        # If we don't start off with a parenthesis, all we have to do
        # is strip off the first element and process it.  We then
        # call the routine over again to handle the next part of
        # the string...
        if (compound_string[0] != '(') :
            # Look to see if the string has an element
            # like "C" or like "He".
            first_char=compound_string[0]
            if len(compound_string) > 1: 
                second_char = compound_string[1]
            else: 
                second_char = ''
            this_element_name = first_char
      
            if ((second_char >= 'a') and (second_char <= 'z')) :
                this_element_name = this_element_name + second_char
                num_start_index = 2
            else:
                this_element_name = this_element_name + ' '
                num_start_index = 1
      
        if verbose: print ('this_element_name:',this_element_name,', num_start_index:', num_start_index)
        
        this_z=0
        if   this_element_name == 'H ': this_z=1
        elif this_element_name == 'He': this_z=2
        elif this_element_name == 'Li': this_z=3
        elif this_element_name == 'Be': this_z=4
        elif this_element_name == 'B ': this_z=5
        elif this_element_name == 'C ': this_z=6
        elif this_element_name == 'N ': this_z=7
        elif this_element_name == 'O ': this_z=8
        elif this_element_name == 'F ': this_z=9
        elif this_element_name == 'Ne': this_z=10
        elif this_element_name == 'Na': this_z=11
        elif this_element_name == 'Mg': this_z=12
        elif this_element_name == 'Al': this_z=13
        elif this_element_name == 'Si': this_z=14
        elif this_element_name == 'P ': this_z=15
        elif this_element_name == 'S ': this_z=16
        elif this_element_name == 'Cl': this_z=17
        elif this_element_name == 'Ar': this_z=18
        elif this_element_name == 'K ': this_z=19
        elif this_element_name == 'Ca': this_z=20
        elif this_element_name == 'Sc': this_z=21
        elif this_element_name == 'Ti': this_z=22
        elif this_element_name == 'V ': this_z=23
        elif this_element_name == 'Cr': this_z=24
        elif this_element_name == 'Mn': this_z=25
        elif this_element_name == 'Fe': this_z=26
        elif this_element_name == 'Co': this_z=27
        elif this_element_name == 'Ni': this_z=28
        elif this_element_name == 'Cu': this_z=29
        elif this_element_name == 'Zn': this_z=30
        elif this_element_name == 'Ga': this_z=31
        elif this_element_name == 'Ge': this_z=32
        elif this_element_name == 'As': this_z=33
        elif this_element_name == 'Se': this_z=34
        elif this_element_name == 'Br': this_z=35
        elif this_element_name == 'Kr': this_z=36
        elif this_element_name == 'Rb': this_z=37
        elif this_element_name == 'Sr': this_z=38
        elif this_element_name == 'Y ': this_z=39
        elif this_element_name == 'Zr': this_z=40
        elif this_element_name == 'Nb': this_z=41
        elif this_element_name == 'Mo': this_z=42
        elif this_element_name == 'Tc': this_z=43
        elif this_element_name == 'Ru': this_z=44
        elif this_element_name == 'Rh': this_z=45
        elif this_element_name == 'Pd': this_z=46
        elif this_element_name == 'Ag': this_z=47
        elif this_element_name == 'Cd': this_z=48
        elif this_element_name == 'In': this_z=49
        elif this_element_name == 'Sn': this_z=50
        elif this_element_name == 'Sb': this_z=51
        elif this_element_name == 'Te': this_z=52
        elif this_element_name == 'I ': this_z=53
        elif this_element_name == 'Xe': this_z=54
        elif this_element_name == 'Cs': this_z=55
        elif this_element_name == 'Ba': this_z=56
        elif this_element_name == 'La': this_z=57
        elif this_element_name == 'Ce': this_z=58
        elif this_element_name == 'Pr': this_z=59
        elif this_element_name == 'Nd': this_z=60
        elif this_element_name == 'Pm': this_z=61
        elif this_element_name == 'Sm': this_z=62
        elif this_element_name == 'Eu': this_z=63
        elif this_element_name == 'Gd': this_z=64
        elif this_element_name == 'Tb': this_z=65
        elif this_element_name == 'Dy': this_z=66
        elif this_element_name == 'Ho': this_z=67
        elif this_element_name == 'Er': this_z=68
        elif this_element_name == 'Tm': this_z=69
        elif this_element_name == 'Yb': this_z=70
        elif this_element_name == 'Lu': this_z=71
        elif this_element_name == 'Hf': this_z=72
        elif this_element_name == 'Ta': this_z=73
        elif this_element_name == 'W ': this_z=74
        elif this_element_name == 'Re': this_z=75
        elif this_element_name == 'Os': this_z=76
        elif this_element_name == 'Ir': this_z=77
        elif this_element_name == 'Pt': this_z=78
        elif this_element_name == 'Au': this_z=79
        elif this_element_name == 'Hg': this_z=80
        elif this_element_name == 'Tl': this_z=81
        elif this_element_name == 'Pb': this_z=82
        elif this_element_name == 'Bi': this_z=83
        elif this_element_name == 'Po': this_z=84
        elif this_element_name == 'At': this_z=85
        elif this_element_name == 'Rn': this_z=86
        elif this_element_name == 'Fr': this_z=87
        elif this_element_name == 'Ra': this_z=88
        elif this_element_name == 'Ac': this_z=89
        elif this_element_name == 'Th': this_z=90
        elif this_element_name == 'Pa': this_z=91
        elif this_element_name == 'U ': this_z=92
        else: this_z=0
        
            
        if (this_z == 0) :
            print ('zcompound is confused: ',compound_string)
            compound_string=''
            z_array = 0
            return
        
        
        
        # Find the next element or parenthesis, as
        # anything before it must be a number.
        postnum_index = num_start_index
        if len(compound_string) > num_start_index+1 :
            test_char = compound_string[postnum_index]
        else :
            test_char = ''
        while ( ((test_char == '0') or (test_char == '1') or \
                 (test_char == '2') or (test_char == '2') or \
                 (test_char == '3') or (test_char == '4') or \
                 (test_char == '5') or (test_char == '6') or \
                 (test_char == '7') or (test_char == '8') or \
                 (test_char == '9') or (test_char == '.')) and \
                 (postnum_index <= last_char_index) ) :
            postnum_index=postnum_index+1
            if (postnum_index <= last_char_index) :
                test_char=compound_string[postnum_index]
            else:
                test_char=''
              
        # is there more?
        if (num_start_index != postnum_index) :
            number_string=compound_string[num_start_index:postnum_index]
            num_multiplier=1.
            if verbose: print( 'Trying to interpret ',number_string,' as a number.')
            if (len(number_string) != 0) :
                num_multiplier = float(number_string)
        else:
            num_multiplier=1.

        # We've handled this element, so pop it into the
        # matrix and continue.
        if (this_z <= max_z_index) :
            z_array[this_z-1] = z_array[this_z-1] + num_multiplier 
        else:
            print ('zcompound: z_array smaller than ',max_z_index)
            z_array = 0
            return

        # And deal with what's left
        remaining_string=compound_string[postnum_index:last_char_index+1]
 
        if len(remaining_string) > 0:
            z_array = self.zcompound(remaining_string,z_array,paren_multiplier=True)
        
        return z_array
    
    
#-----------------------------------------------------------------------------    
    def zatwt(self, z_array): 
        
        maxz=z_array.size
        atwt=0.

        for i in range(maxz):
            if (z_array[i] != 0.):
                if   i+1 == 1: this_atwt=1.00794
                elif i+1 == 2: this_atwt=4.0026
                elif i+1 == 3: this_atwt=6.941
                elif i+1 == 4: this_atwt=9.01218
                elif i+1 == 5: this_atwt=10.81
                elif i+1 == 6: this_atwt=12.011
                elif i+1 == 7: this_atwt=14.0067
                elif i+1 == 8: this_atwt=15.9994
                elif i+1 == 9: this_atwt=18.9984
                elif i+1 == 10: this_atwt=21.179
                elif i+1 == 11: this_atwt=22.98977
                elif i+1 == 12: this_atwt=24.305
                elif i+1 == 13: this_atwt=26.98154
                elif i+1 == 14: this_atwt=28.0855
                elif i+1 == 15: this_atwt=30.97376
                elif i+1 == 16: this_atwt=32.06
                elif i+1 == 17: this_atwt=35.453
                elif i+1 == 18: this_atwt=39.948
                elif i+1 == 19: this_atwt=39.0983
                elif i+1 == 20: this_atwt=40.08
                elif i+1 == 21: this_atwt=44.9559
                elif i+1 == 22: this_atwt=47.88
                elif i+1 == 23: this_atwt=50.9415
                elif i+1 == 24: this_atwt=51.996
                elif i+1 == 25: this_atwt=54.9380
                elif i+1 == 26: this_atwt=55.847
                elif i+1 == 27: this_atwt=58.9332
                elif i+1 == 28: this_atwt=58.69
                elif i+1 == 29: this_atwt=63.546
                elif i+1 == 30: this_atwt=65.38
                elif i+1 == 31: this_atwt=69.72
                elif i+1 == 32: this_atwt=72.59
                elif i+1 == 33: this_atwt=74.9216
                elif i+1 == 34: this_atwt=78.96
                elif i+1 == 35: this_atwt=79.904
                elif i+1 == 36: this_atwt=83.80
                elif i+1 == 37: this_atwt=85.4678
                elif i+1 == 38: this_atwt=87.62
                elif i+1 == 39: this_atwt=88.9059
                elif i+1 == 40: this_atwt=91.22
                elif i+1 == 41: this_atwt=92.9064
                elif i+1 == 42: this_atwt=95.94
                elif i+1 == 43: this_atwt=98.
                elif i+1 == 44: this_atwt=101.07
                elif i+1 == 45: this_atwt=102.9055
                elif i+1 == 46: this_atwt=106.42
                elif i+1 == 47: this_atwt=107.8682
                elif i+1 == 48: this_atwt=112.41
                elif i+1 == 49: this_atwt=114.82
                elif i+1 == 50: this_atwt=118.69
                elif i+1 == 51: this_atwt=121.75
                elif i+1 == 52: this_atwt=127.60
                elif i+1 == 53: this_atwt=126.9054
                elif i+1 == 54: this_atwt=131.29
                elif i+1 == 55: this_atwt=132.9054
                elif i+1 == 56: this_atwt=137.33
                elif i+1 == 57: this_atwt=138.9055
                elif i+1 == 58: this_atwt=140.12
                elif i+1 == 59: this_atwt=140.9077
                elif i+1 == 60: this_atwt=144.24
                elif i+1 == 61: this_atwt=145.
                elif i+1 == 62: this_atwt=150.36
                elif i+1 == 63: this_atwt=151.96
                elif i+1 == 64: this_atwt=157.25
                elif i+1 == 65: this_atwt=158.9254
                elif i+1 == 66: this_atwt=162.5
                elif i+1 == 67: this_atwt=164.9304
                elif i+1 == 68: this_atwt=167.26
                elif i+1 == 69: this_atwt=168.9342
                elif i+1 == 70: this_atwt=173.04
                elif i+1 == 71: this_atwt=174.967
                elif i+1 == 72: this_atwt=178.49
                elif i+1 == 73: this_atwt=180.9479
                elif i+1 == 74: this_atwt=183.85
                elif i+1 == 75: this_atwt=186.207
                elif i+1 == 76: this_atwt=190.2
                elif i+1 == 77: this_atwt=192.22
                elif i+1 == 78: this_atwt=195.08
                elif i+1 == 79: this_atwt=196.9665
                elif i+1 == 80: this_atwt=200.59
                elif i+1 == 81: this_atwt=204.383
                elif i+1 == 82: this_atwt=207.2
                elif i+1 == 83: this_atwt=208.9804
                elif i+1 == 84: this_atwt=209.
                elif i+1 == 85: this_atwt=210.
                elif i+1 == 86: this_atwt=222.
                elif i+1 == 87: this_atwt=223.
                elif i+1 == 88: this_atwt=226.0254
                elif i+1 == 89: this_atwt=227.0278
                elif i+1 == 90: this_atwt=232.0381
                elif i+1 == 91: this_atwt=231.0359
                elif i+1 == 92: this_atwt=238.0289
                else: this_atwt=0.

                atwt=atwt+z_array[i]*this_atwt

        return atwt


#-----------------------------------------------------------------------------    
    def extra(self, ielement = -1): 
        
        energies, f1, f2, n_extra, energies_extra, f1_extra, f2_extra = self.read(ielement, all = False)
        
        if n_extra != 0:
            energies_all=np.concatenate((energies,energies_extra), axis=0)
            f1_all=np.concatenate((f1,f1_extra), axis=0)
            f2_all=np.concatenate((f2,f2_extra), axis=0)
            sort_order=energies_all.argsort()
            energies_all=energies_all[sort_order]
            f1_all=f1_all[sort_order]
            f2_all=f2_all[sort_order]
        else:
            energies_all=energies
            f1_all=f1
            f2_all=f2


        return energies, f1, f2, energies_extra, f1_extra, f2_extra

#-----------------------------------------------------------------------------    
    def read(self, ielement = -1, all = True): 
        
        
        # If we don't specifiy element return all energies 
        if ielement == -1 : 
            all = True
        
        verbose = False
        expected_pos = 0
        
        filename = 'henke.xdr'
        file = open(str(filename),'rb')
#         try:
#             file = open(str(filename),'rb')
#         except:
#             print ('Could not open file ', filename)
#             return -1
        
        if verbose:
            print ('File: ', filename)
    
        buf = file.read()        
        u = Unpacker(buf)
        
        if all:
            n_elements = u.unpack_int()
            n_energies = u.unpack_int()
        
            if verbose:
                print ('n_energies: ', n_energies)
                print ('n_elements: ', n_elements)
                expected_pos = expected_pos+2*4
                print ('Actual, expected file position before reading energies: ' ,u.get_position(), expected_pos)
            
            
            energies = u.unpack_farray(n_energies, u.unpack_float)   
            energies = np.array(energies) 
            if verbose:
                print ('energies: ', energies)
            
            f1 = np.zeros((n_elements, n_energies))
            f2 = np.zeros((n_elements, n_energies))
            this_f1 = np.zeros((n_energies))
            this_f2 = np.zeros((n_energies))

            if verbose:        
                expected_pos = expected_pos+4*n_energies
                print ('Actual, expected file position before reading elements: ',u.get_position(), expected_pos)
        
            for i_element in range(n_elements):
                this_f1 = u.unpack_farray(n_energies, u.unpack_float)
                this_f2 = u.unpack_farray(n_energies, u.unpack_float)
                f1[i_element, :] = this_f1
                f2[i_element, :] = this_f2
            
            if verbose:
                expected_pos =expected_pos+n_elements*n_energies*2*4
                print ('Actual, expected file position before reading n_extra_energies: ', u.get_position(), expected_pos)

            n_extra_energies  = u.unpack_int()
            if verbose:
                print ('n_extra_energies: ', n_extra_energies)
            
            if verbose:        
                expected_pos = expected_pos+4
                print ('Actual, expected file position before reading extras: ',u.get_position(), expected_pos)
            
            
            n_extra = np.zeros((n_elements), dtype = np.int)
            extra_energies = np.zeros((n_elements, n_extra_energies))
            extra_f1 = np.zeros((n_elements, n_extra_energies))
            extra_f2 = np.zeros((n_elements, n_extra_energies))
            this_n_extra = 0
            this_extra_energies = np.zeros((n_extra_energies))
            this_extra_f1 = np.zeros((n_extra_energies))
            this_extra_f2 = np.zeros((n_extra_energies))
        
            for i_element in range(n_elements):
                this_n_extra = u.unpack_int()
                this_extra_energies = u.unpack_farray(n_extra_energies, u.unpack_float)
                this_extra_f1 = u.unpack_farray(n_extra_energies, u.unpack_float)
                this_extra_f2 = u.unpack_farray(n_extra_energies, u.unpack_float)
                n_extra[i_element] = this_n_extra
                extra_energies[i_element, :] = this_extra_energies
                extra_f1[i_element, :] = this_extra_f1
                extra_f2[i_element, :] = this_extra_f2
                
        else:
            n_elements = u.unpack_int()
            n_energies = u.unpack_int()           
            
            energies = u.unpack_farray(n_energies, u.unpack_float)   
            energies = np.array(energies)  
            if verbose:
                print ('energies: ', energies)
            
            byte_offset = 4+4+4*n_energies + 8*ielement*n_energies
            u.set_position(byte_offset)
            
            f1 = u.unpack_farray(n_energies, u.unpack_float)
            f2 = u.unpack_farray(n_energies, u.unpack_float)
            
            byte_offset = 4+4+4*n_energies + 8*n_elements*n_energies
            u.set_position(byte_offset)
            
            n_extra_energies = u.unpack_int()
            if verbose: print ('n_extra_energies ', n_extra_energies)
            
            
            # Now we have the above plus i_element times the quantity:
            #   (2 for n_extra, and n_extra_energies each of three floats)
            byte_offset = long(4)+long(4)+long(4)*n_energies + long(8)*n_elements*n_energies + long(4) + ielement*(4+12*n_extra_energies)
            u.set_position(byte_offset)
            
            n_extra = u.unpack_int()
            this_extra_energies = u.unpack_farray(n_extra_energies, u.unpack_float)
            this_extra_f1 = u.unpack_farray(n_extra_energies, u.unpack_float)
            this_extra_f2 = u.unpack_farray(n_extra_energies, u.unpack_float)
            
            extra_energies = this_extra_energies[0:n_extra]
            extra_f1 = this_extra_f1[0:n_extra]
            extra_f2 = this_extra_f2[0:n_extra]
                                                
        file.close()
        
        
        return energies, f1, f2, n_extra, extra_energies, extra_f1, extra_f2
    
    
    
#-----------------------------------------------------------------------------    
    def array(self, compound_name, density, graze_mrad = 0):

        z_array, atwt = self.compound(compound_name,density)

        maxz = 92
        first_time = 1
        for i in range(maxz):
            if (z_array[i] != 0.):
                energies, this_f1, this_f2, n_extra, extra_energies, extra_f1, extra_f2 = self.read(ielement=i)
            if (first_time == 1) :
                f1 = z_array[i]*this_f1
                f2 = z_array[i]*this_f2
                first_time = 0
            else:
                f1 = f1+z_array[i]*this_f1
                f2 = f2+z_array[i]*this_f2

  
        num_energies = len(energies)
        AVOGADRO=6.02204531e23
        HC_ANGSTROMS=12398.52
        RE=2.817938070e-13            # in cm
  
        if (atwt != 0.0) :
            molecules_per_cc = density * AVOGADRO / atwt
        else:
            molecules_per_cc = 0.0

  
        wavelength_angstroms = HC_ANGSTROMS/energies
        # This constant has wavelength in angstroms and then
        # they are converted to centimeters.
        constant = RE * (1.0e-16 * wavelength_angstroms * wavelength_angstroms) * molecules_per_cc / (2.0 * np.math.pi)
        delta = constant * f1
        beta = constant * f2
        # Alpha is in inverse meters squared
        alpha = 1.e4 * density * AVOGADRO * RE / (2.*np.math.pi*atwt)
        #alpha = alpha[0]

  
        if (graze_mrad == 0.):
            reflect=np.ones((num_energies))
        else:
            theta = 1.0e-3 * graze_mrad
            sinth = np.sin(theta)
            sinth2 = sinth * sinth
            coscot = np.cos(theta)
            coscot = coscot * coscot / sinth
            alpha = 2.0 * delta - delta * delta + beta * beta
            gamma = 2.0 * (1.0 - delta) * beta
            rhosq = 0.5 * (sinth2 - alpha + np.sqrt((sinth2 - alpha)*(sinth2-alpha) + gamma*gamma) )
            rho = np.sqrt(rhosq)
            i_sigma = (4.0 * rhosq * (sinth - rho) * (sinth - rho) + \
                 gamma * gamma) / \
                 (4.0 * rhosq * (sinth + rho) * (sinth + rho) + \
                  gamma * gamma)
            piosig = (4.0 * rhosq * (rho - coscot) * (rho - coscot) + \
                gamma * gamma) / \
                (4.0 * rhosq * (rho + coscot) * (rho + coscot) + \
                 gamma * gamma)
            reflect= 50.0 * i_sigma * (1 + piosig)


        denom = energies*4.*np.math.pi*beta

        zeroes = np.where(denom == 0.)
        nonzeroes = np.where(denom != 0.)  
        denom[zeroes] = 1e-8     
        
        inverse_mu = np.array((len(energies)))

        inverse_mu = 1.239852/denom
        if (len(zeroes) > 0) :
            inverse_mu[zeroes] = np.inf


        return energies, f1, f2, delta, beta, graze_mrad, reflect, inverse_mu, atwt, alpha
  


#-----------------------------------------------------------------------------    
    def dose_calc(self, stack, i_composition, od_spectrum, i0_signal, dosecalc_detector_eff):    
    
        pix_nm_squared = 1.e6*(stack.x_dist[1]-stack.x_dist[0])*(stack.y_dist[1]-stack.y_dist[0])

        dose = 0.
             

        #rho doesn't matter here
        rho = 1.
        henke_energies, f1, f2, delta, beta, graze_mrad, reflect, inverse_mu, atwt, alpha = self.array(i_composition,rho) 

        func_f2_array = scipy.interpolate.interp1d(henke_energies, f2,  bounds_error=False, fill_value=0.0) 
        f2_array = func_f2_array(stack.ev)
        
        # This is the scaling of f2_array as described in pca_gui_man.tex
        i_max_ev = np.argmax(stack.ev)
        if stack.data_dwell == None: 
            stack.data_dwell = 1.
        if (od_spectrum[i_max_ev] != 0.) :
            f2_array = f2_array[0,i_max_ev]*(stack.ev/stack.ev[i_max_ev])*(od_spectrum/od_spectrum[i_max_ev])
            # See the documentation file pca_gui_man.tex. Is i0_signal
            # normalized to 1 sec? Does not seem to be the case! Might
            # have to change it at some point.
            dose  = 6.74e5*(1./(dosecalc_detector_eff* pix_nm_squared*atwt))*np.sum(i0_signal*1e-3*stack.data_dwell *f2_array)


        return dose



# 
#   This file is part of Mantis, a Multivariate ANalysis Tool for Spectromicroscopy.
# 
#   Copyright (C) 2011 Mirna Lerotic, 2nd Look
#   http://2ndlookconsulting.com
#   License: GNU GPL v3
#
#   Mantis is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version.
#
#   Mantis is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details <http://www.gnu.org/licenses/>.


from __future__ import division
import os.path
import numpy as np
import scipy as scy
import netCDF4 

from mantis_xray import data_struct


#----------------------------------------------------------------------    
def read_sm_header(filename):
    
    try:
        print (filename)
        f = netCDF4.Dataset(filename, 'r', format='NETCDF4')

        
        ncols = len(f.dimensions['n_fast_pixels'])
        nrows = len(f.dimensions['n_slow_pixels'])
        ev = getattr(f,'start_ev')
        x_center_um = getattr(f,'x_center_um')
        y_center_um = getattr(f,'y_center_um')


        f.close()
    
        return 1, ncols, nrows, ev
    except:
        return -1, 0,0,0
        
#----------------------------------------------------------------------    
def read_sm_list(self, filelist, filepath, ds): 
    
    #Fill the common stack data 
    file1 = os.path.join(filepath, filelist[0])   
    fcdf = netCDF4.Dataset(file1, 'r', format='NETCDF4')   

        
    ncols = len(fcdf.dimensions['n_fast_pixels'])
    nrows = len(fcdf.dimensions['n_slow_pixels'])
    npix = ncols*nrows
    nev = len(filelist)
    absdata = np.zeros((ncols, nrows, nev))
    ev = np.zeros((nev))
    
    #fdevicen = netCDF4.chartostring(getattr(fcdf,'fast_device_name'))
    #sdevicen = netCDF4.chartostring(getattr(fcdf,'slow_device_name'))
    
    fdevicenl = getattr(fcdf,'fast_device_name')
    sdevicenl = getattr(fcdf,'slow_device_name')
    
    fdevicen = chr(fdevicenl[0]) + chr(fdevicenl[1]) + chr(fdevicenl[2]) + chr(fdevicenl[3])
    sdevicen = chr(sdevicenl[0]) + chr(sdevicenl[1]) + chr(sdevicenl[2]) + chr(sdevicenl[3])
    
    x_center_um = -1
    y_center_um = -1
    if fdevicen == 'XPZT':
        x_center_um = getattr(fcdf,'x_center_um')
    if sdevicen == 'YPZT':
        y_center_um = getattr(fcdf,'y_center_um')   
        
    if (x_center_um == -1) or (y_center_um == -1):
        print ('Error in stack build: Not piezo scan')
        return
    
    x_pixel_um = getattr(fcdf,'fast_pixel_um')
    y_pixel_um = getattr(fcdf,'slow_pixel_um')
    
    x_direction = getattr(fcdf,'fast_direction')
    y_direction = getattr(fcdf,'slow_direction')
    
    dwell_msec = getattr(fcdf,'dwell_msec')
            
    x_dist = (np.arange(npix)-np.fix(npix/2.0)) * x_direction * x_pixel_um + x_center_um
    y_dist = (np.arange(npix)-np.fix(npix/2.0)) * y_direction * y_pixel_um + y_center_um         
    
    
    ds.implements = 'information:exchange:spectromicroscopy'
    ds.version = '1.0'
    ds.information.file_creation_datetime = ' '#netCDF4.chartostring(getattr(fcdf,'systime'))
    ds.information.comment = ' '#netCDF4.chartostring(getattr(fcdf,'comments1'))
    
    ds.information.experimenter.name = ' '#netCDF4.chartostring(getattr(fcdf,'scientist'))

    ds.information.sample.name = ' '#netCDF4.chartostring(getattr(fcdf,'sample'))
    
    
    fcdf.close()
    
    
    
    #Read the image data        
    for i in range(len(filelist)):
        fn = filelist[i]
        filename = os.path.join(filepath, fn)   
        fcdf = netCDF4.Dataset(filename, 'r', format='NETCDF4')   
        
        image = fcdf.variables['image_data'][:]
        absdata[:,:,i] = image[:,:,0]
        eng = getattr(fcdf,'start_ev')
        ev[i] = eng
            
        fcdf.close()
        
    if ev[-1]<ev[0]:
        ev = ev[::-1]
        absdata = absdata[:,:, ::-1]
        
    
    ds.exchange.data = absdata
    ds.exchange.data_signal = 1
    ds.exchange.data_axes='x:y'
    
    ds.exchange.energy=ev
    ds.exchange.energy_units = 'ev'
    
    
    ds.exchange.x = x_dist
    ds.exchange.x_units = 'um'
    ds.exchange.y = y_dist
    ds.exchange.y_units = 'um'        
    ds.spectromicroscopy.data_dwell = dwell_msec
    
    return

#----------------------------------------------------------------------
class sm:
    def __init__(self, ds):
        self.ds = ds

    
#----------------------------------------------------------------------    
    def read_sm_header(self, filename):
        
        try:
            f = netCDF4.Dataset(filename, 'r', format='NETCDF4')
        
        
#        print f.file_format
#        print f.groups
#        print f.variables
        #print f.dimensions
#        
#        for dimname, dimobj in f.dimensions.items():
#            print dimname, len(dimobj), dimobj.isunlimited()
            
            ncols = len(f.dimensions['n_fast_pixels'])
            nrows = len(f.dimensions['n_slow_pixels'])
            ev = getattr(f,'start_ev')
            x_center_um = getattr(f,'x_center_um')
            y_center_um = getattr(f,'y_center_um')
            
#        
#        for name in f.ncattrs():
#            print 'Global attr', name, '=', getattr(f,name)
            
        #print netCDF4.chartostring(getattr(f,'detector_name'))
        
        #n_cols = getattr(f, )
  
            f.close()
        
            return 1, ncols, nrows, ev
        except:
            return -1, 0,0,0
        
#----------------------------------------------------------------------    
    def read_sm_list(self, filelist, filepath, ds): 
        
        #Fill the common stack data 
        file1 = os.path.join(filepath, filelist[0])   
        fcdf = netCDF4.Dataset(file1, 'r', format='NETCDF4')   

            
        ncols = len(fcdf.dimensions['n_fast_pixels'])
        nrows = len(fcdf.dimensions['n_slow_pixels'])
        npix = ncols*nrows
        nev = len(filelist)
        absdata = np.zeros((ncols, nrows, nev))
        ev = np.zeros((nev))
        
        #fdevicen = netCDF4.chartostring(getattr(fcdf,'fast_device_name'))
        #sdevicen = netCDF4.chartostring(getattr(fcdf,'slow_device_name'))
        
        fdevicenl = getattr(fcdf,'fast_device_name')
        sdevicenl = getattr(fcdf,'slow_device_name')
        
        fdevicen = chr(fdevicenl[0]) + chr(fdevicenl[1]) + chr(fdevicenl[2]) + chr(fdevicenl[3])
        sdevicen = chr(sdevicenl[0]) + chr(sdevicenl[1]) + chr(sdevicenl[2]) + chr(sdevicenl[3])
        
        x_center_um = -1
        y_center_um = -1
        if fdevicen == 'XPZT':
            x_center_um = getattr(fcdf,'x_center_um')
        if sdevicen == 'YPZT':
            y_center_um = getattr(fcdf,'y_center_um')   
            
        if (x_center_um == -1) or (y_center_um == -1):
            print ('Error in stack build: Not piezo scan')
            return
        
        x_pixel_um = getattr(fcdf,'fast_pixel_um')
        y_pixel_um = getattr(fcdf,'slow_pixel_um')
        
        x_direction = getattr(fcdf,'fast_direction')
        y_direction = getattr(fcdf,'slow_direction')
        
        dwell_msec = getattr(fcdf,'dwell_msec')
                
        x_dist = (np.arange(npix)-np.fix(npix/2.0)) * x_direction * x_pixel_um + x_center_um
        y_dist = (np.arange(npix)-np.fix(npix/2.0)) * y_direction * y_pixel_um + y_center_um         
        
        
        self.ds.implements = 'information:exchange:spectromicroscopy'
        self.ds.version = '1.0'
        self.ds.information.file_creation_datetime = ' '#netCDF4.chartostring(getattr(fcdf,'systime'))
        self.ds.information.comment = ' '#netCDF4.chartostring(getattr(fcdf,'comments1'))
        
        self.ds.information.experimenter.name = ' '#netCDF4.chartostring(getattr(fcdf,'scientist'))

        self.ds.information.sample.name = ' '#netCDF4.chartostring(getattr(fcdf,'sample'))
        
        
        fcdf.close()
        
        
        
        #Read the image data        
        for i in range(len(filelist)):
            fn = filelist[i]
            filename = os.path.join(filepath, fn)   
            fcdf = netCDF4.Dataset(filename, 'r', format='NETCDF4')   
            
            image = fcdf.variables['image_data'][:]
            absdata[:,:,i] = image[:,:,0]
            eng = getattr(fcdf,'start_ev')
            ev[i] = eng
                
            fcdf.close()
            
        if ev[-1]<ev[0]:
            ev = ev[::-1]
            absdata = absdata[:,:, ::-1]
            
        
        self.ds.exchange.data = absdata
        self.ds.exchange.data_signal = 1
        self.ds.exchange.data_axes='x:y'
        
        self.ds.exchange.energy=ev
        self.ds.exchange.energy_units = 'ev'
        
        
        self.ds.exchange.x = x_dist
        self.ds.exchange.x_units = 'um'
        self.ds.exchange.y = y_dist
        self.ds.exchange.y_units = 'um'        
        self.ds.spectromicroscopy.data_dwell = dwell_msec
        
        return



import setuptools
import mantis_xray

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mantis-xray", 
    version=mantis_xray.__version__,
    author="Mirna Lerotic",
    author_email="mirna@2ndlookconsulting.com",
    description="MANTiS is a Multivariate ANalysis Tool for x-ray Spectromicroscopy",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://spectromicroscopy.com/",
    project_urls={
        "Code": "https://github.com/mlerotic/spectromicroscopy",
        "Documentation": "https://docs.spectromicroscopy.com",
    },
    install_requires=['PyQt5','numpy', 'scipy', 'matplotlib', 'h5py', 'Pillow', 'lxml', 'pyqtgraph<=0.12.4', "scikit-image"],
    extras_require={
        "netCDF":  "netcdf4-python"},
    entry_points={
        "gui_scripts": "mantis-xray = mantis_xray.mantis_qt:main"},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
    ],
    packages=setuptools.find_packages(),
    include_package_data=True,
    python_requires=">=3",
)



# 
#   This file is part of Mantis, a Multivariate ANalysis Tool for Spectromicroscopy.
# 
#   Copyright (C) 2011 Mirna Lerotic, 2nd Look
#   http://2ndlookconsulting.com
#   License: GNU GPL v3
#
#   Mantis is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version.
#
#   Mantis is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details <http://www.gnu.org/licenses/>.


from __future__ import division

import numpy as np
import scipy.interpolate
import os



title = 'STK'
extension = ['*.stk']
read_types = ['spectrum','image','stack']
write_types = []


def identify(filename):
        
    try:
        #Binary file - the only way to check if the file is ok to check the extension
        bn, extension = os.path.splitext(filename) 
        if extension.lower() == '.stk':
            return True
        else:
            return False
    except:
        return False


def read_old( filepath, data_stk):
    data_stk.read_h5(filepath)
    return


def GetFileStructure(FileName):
    return None


#----------------------------------------------------------------------
def read(filename, self, selection=None, *args, **kwargs):
    f = open(str(filename),'rb')
    data = np.fromfile(f, np.int32, 3)
    data.byteswap(True)
    
    self.n_cols = data[0]
    self.n_rows = data[1]
    self.n_ev = data[2]
    
    #print 'self.n_cols, self.n_rows, self.n_ev', self.n_cols, self.n_rows, self.n_ev
    
    self.x_dist = np.fromfile(f, np.float32, self.n_cols)
    self.x_dist.byteswap(True)   
    
    
    self.y_dist = np.fromfile(f, np.float32, self.n_rows)
    self.y_dist.byteswap(True)     
    

    self.ev = np.fromfile(f, np.float32, self.n_ev)
    self.ev.byteswap(True)   
    
    msec = np.fromfile(f, np.float32, self.n_ev)
    msec.byteswap(True) 
     
    self.data_dwell = msec
    
    imagestack = np.fromfile(f, np.float32, self.n_cols*self.n_rows*self.n_ev)
    imagestack.byteswap(True)     
           
    self.absdata = np.empty((self.n_cols, self.n_rows, self.n_ev))
            
    self.absdata = np.reshape(imagestack, (self.n_cols, self.n_rows, self.n_ev), order='F')       

    f.close()
    
    self.fill_h5_struct_from_stk()
    
  
    return

#----------------------------------------------------------------------   
def read_stk_i0_xas(self, filename):

    f = open(str(filename),'r')
    
    elist = []
    ilist = []    

    for line in f:
        if line.startswith("*"):
            pass
        else:
            e, i = [float (x) for x in line.split()] 
            elist.append(e)
            ilist.append(i)
            
    self.evi0 = np.array(elist)
    self.i0data = np.array(ilist) 
            
    f.close()
    
    self.i0_dwell = None
       

    return
    

#----------------------------------------------------------------------   
def read_stk_i0_csv(self, filename):

    f = open(str(filename),'r')
    
    elist = []
    ilist = []    

    for line in f:
        if line.startswith("*"):
            pass
        else:
            e, i = [float (x) for x in line.split(',')] 
            elist.append(e)
            ilist.append(i)
            
    self.evi0 = np.array(elist)
    self.i0data = np.array(ilist) 
            
    f.close()
    
    self.i0_dwell = None
       

    return    
    
#----------------------------------------------------------------------
class x1astk:
    def __init__(self):
        pass
    
#----------------------------------------------------------------------
    def read_stk(self, filename):
        f = open(str(filename),'rb')
        data = np.fromfile(f, np.int32, 3)
        data.byteswap(True)
        
        self.n_cols = data[0]
        self.n_rows = data[1]
        self.n_ev = data[2]
        
        #print 'self.n_cols, self.n_rows, self.n_ev', self.n_cols, self.n_rows, self.n_ev
        
        self.x_dist = np.fromfile(f, np.float32, self.n_cols)
        self.x_dist.byteswap(True)   
        
        
        self.y_dist = np.fromfile(f, np.float32, self.n_rows)
        self.y_dist.byteswap(True)     
        

        self.ev = np.fromfile(f, np.float32, self.n_ev)
        self.ev.byteswap(True)   
        
        msec = np.fromfile(f, np.float32, self.n_ev)
        msec.byteswap(True) 
         
        self.data_dwell = msec
        
        imagestack = np.fromfile(f, np.float32, self.n_cols*self.n_rows*self.n_ev)
        imagestack.byteswap(True)     
               
        self.absdata = np.empty((self.n_cols, self.n_rows, self.n_ev))
                
        self.absdata = np.reshape(imagestack, (self.n_cols, self.n_rows, self.n_ev), order='F')       

        f.close()
        
#         self.original_n_cols = self.n_cols.copy()
#         self.original_n_rows = self.n_rows.copy()
#         self.original_n_ev = self.n_ev.copy()
#         self.original_ev = self.ev.copy()
#         self.original_absdata = self.absdata.copy()
      
        return

#----------------------------------------------------------------------   
    def read_stk_i0_xas(self, filename):

        f = open(str(filename),'r')
        
        elist = []
        ilist = []    
    
        for line in f:
            if line.startswith("*"):
                pass
            else:
                e, i = [float (x) for x in line.split()] 
                elist.append(e)
                ilist.append(i)
                
        self.evi0 = np.array(elist)
        self.i0data = np.array(ilist) 
                
        f.close()
        
        self.i0_dwell = None
           
    
        return
    

#----------------------------------------------------------------------   
    def read_stk_i0_csv(self, filename):

        f = open(str(filename),'r')
        
        elist = []
        ilist = []    
    
        for line in f:
            print(line)
            if line.startswith("*"):
                pass
            else:
                e, i = [float (x) for x in line.split(',')] 
                elist.append(e)
                ilist.append(i)
                
        self.evi0 = np.array(elist)
        self.i0data = np.array(ilist) 
                
        f.close()
        
        self.i0_dwell = None
           
    
        return            
        
        
        
        
        
        
        
       
            
    
